@@ -39,6 +39,7 @@
 
 #include <iostream.h>
 #include <string.h>
+#include <stdio.h>
 #include <ctype.h>
 #include "EST_String.h"
 #include "string_version.h"
@@ -59,7 +60,7 @@ struct subst {
   int slen;
 } ;
 
-#if !defined(__GSUB_REENTRANT__)
+#if !__GSUB_REENTRANT__
 static struct subst *substitutions=NULL;
 int num_substitutions=0;
 #endif
@@ -264,7 +265,7 @@ int EST_String::gsub_internal (const char *os, int olength, const char *s, int l
   const char *from;
   char *to;
   
-#if defined(__GSUB_REENTRANT__)
+#if __GSUB_REENTRANT__
   struct subst {
     EST_String::EST_string_size start, end;
   } *substitutions=NULL;
@@ -327,11 +328,14 @@ int EST_String::gsub_internal (const char *os, int olength, const char *s, int l
   
       if (change > 0)
 	memory = new_memory; 
-      
+	  
+
       size += change;
     }
 
-#if defined(__GSUB_REENTRANT__)
+  //  cout << "gsub n=" << memory.count() << "\n";
+
+#if __GSUB_REENTRANT__
   if (substitutions)
     wfree(substitutions);
 #endif
@@ -354,7 +358,7 @@ int EST_String::gsub_internal (EST_Regex &ex, const char *s, int length)
   const char *from;
   char *to;
   
-#if defined(__GSUB_REENTRANT__)
+#if __GSUB_REENTRANT__
   struct subst *substitutions=NULL;
 
   int num_substitutions=0;
@@ -442,7 +446,7 @@ int EST_String::gsub_internal (EST_Regex &ex, const char *s, int length)
       size += change;
     }
 
-#if defined(__GSUB_REENTRANT__)
+#if __GSUB_REENTRANT__
   if (substitutions)
     wfree(substitutions);
 #endif
@@ -471,16 +475,16 @@ int EST_String::split_internal(EST_String result[], int max,
 	{
 	  start= -1;
 	  end= -1;
-	  if ((*this)[pos] == quote)
+	  if ((*this)(pos) == quote)
 	    {
 	      start=pos;
 	      pos++;
 	      while (pos < length())
 		{
-		  if ((*this)[pos] == quote)
+		  if ((*this)(pos) == quote)
 		    {
 		      pos++;
-		      if ((*this)[pos] != quote)
+		      if ((*this)(pos) != quote)
 			break;
 		      else
 			pos++;
@@ -631,6 +635,23 @@ EST_String operator + (const char *a, const EST_String &b)
     return EST_String(al+bl, c);
 }
 
+EST_String operator * (const EST_String &s, int n)
+{
+
+  if (n<1)
+    return "";
+
+  int l = s.length();
+  int sz = n * l;
+
+  EST_String it(NULL, 0, sz);
+  
+  for(int i=0; i<n; i++)
+    strncpy(((char *)it)+i*l, (const char *)s, l);
+
+  return it;
+}
+
 EST_String &EST_String::operator +=(const char *b) 
 
 {
@@ -677,7 +698,7 @@ EST_String &EST_String::operator += (const EST_String b)
     return *this;
 }
 
-#if !defined(__STRING_INLINE_AGGRESSIVELY__)
+#if !__STRING_INLINE_AGGRESSIVELY__
 EST_String::EST_String(const char *s) 
 {
       CHECK_STRING_ARG(s);
@@ -726,39 +747,47 @@ EST_String::EST_String(const char *s, int start_or_fill, int len)
 
 EST_String::EST_String(const char *s, int s_size, int start, int len) 
 {
-      CHECK_STRING_ARG(s);
+  CHECK_STRING_ARG(s);
       
-      if (len <0)
-	len=s_size-start;
+  if (len <0)
+    len=s_size-start;
 
-      size=len;
-      if (size != 0)
-	memory = chunk_allocate(len+1, s+start, len);
-      else
-	memory=NULL;
-    }
+  size=len;
+  if (size != 0)
+    memory = chunk_allocate(len+1, s+start, len);
+  else
+    memory=NULL;
+}
 
 EST_String::EST_String(const EST_String &s, int start, int len) 
 {
-      if (len <0)
-	len=s.size-start;
+  if (len <0)
+    len=s.size-start;
       
-      size=len;
+  size=len;
 
-      if (start == 0 && len == s.size)
-	memory = NON_CONST_CHUNKPTR(s.memory);
-      else if (size != 0)
-	memory = chunk_allocate(len+1, s.memory, start, len);
-      else
-	memory = NULL;
-    }
-
-EST_String::EST_String(const EST_String &s) 
-{
-    *(struct EST_dumb_string *)this = *(struct EST_dumb_string *)(&s);
+  if (start == 0 && len == s.size)
+    memory = NON_CONST_CHUNKPTR(s.memory);
+  else if (size != 0)
+    memory = chunk_allocate(len+1, s.memory, start, len);
+  else
+    memory = NULL;
 }
 
-#if defined(__FSF_COMPATIBILITY__)
+/*
+EST_String::EST_String(const EST_String &s) 
+{
+#if 1
+  static EST_ChunkPtr hack = NON_CONST_CHUNKPTR(s.memory);
+  memory = NON_CONST_CHUNKPTR(s.memory);
+  size = s.size;
+#else
+    *(struct EST_dumb_string *)this = *(struct EST_dumb_string *)(&s);
+#endif
+}
+*/
+
+#if __FSF_COMPATIBILITY__
 EST_String::EST_String(const char c) 
 {
       size=1;
@@ -789,9 +818,13 @@ EST_String &EST_String::operator = (const char c)
 
 EST_String &EST_String::operator = (const EST_String &s) 
 {
+#if 1
+  static EST_ChunkPtr hack = s.memory;
+  memory = NON_CONST_CHUNKPTR(s.memory);
+  size = s.size;
+#else
       *(struct EST_dumb_string *)this = *(struct EST_dumb_string *)(&s);
-	// memory = NON_CONST_CHUNKPTR(s.memory);
-	// size=s.size;
+#endif
       return *this;
 }
 #endif    
@@ -802,10 +835,10 @@ EST_String downcase(const EST_String &s)
     int i;
 
     for (i=0; i < s.length(); i++)
-	if (isupper(s[i]))
-	    t(i) = tolower(s[i]);
+	if (isupper(s(i)))
+	    t[i] = tolower(s(i));
 	else
-	    t(i) = s[i];
+	    t[i] = s(i);
     return t;
 }	
 
@@ -815,10 +848,10 @@ EST_String upcase(const EST_String &s)
     int i;
 
     for (i=0; i < s.length(); i++)
-	if (islower(s[i]))
-	    t(i) = toupper(s[i]);
+	if (islower(s(i)))
+	    t[i] = toupper(s(i));
 	else
-	    t(i) = s[i];
+	    t[i] = s(i);
     return t;
 }	
 
@@ -894,10 +927,18 @@ EST_String EST_String::unquote(const char quotec) const
 
   result.gsub(quotequote+0, quotequote+1);
 
-  // cout << "after unqote '" << result << "'\n";
+  //  cout << "after unqote '" << result << "'\n";
 
   if (result[0] == quotec && result[result.length()-1] == quotec )
-    return result.at(1, result.length()-2);
+    {
+#if 1
+      /* Spurious local variable to get arounf SunCC 4.0 being broken */
+      EST_String res= result.at(1, result.length()-2);
+      return res;
+#else
+      return result.at(1, result.length()-2);
+#endif
+    }
   else
     return result;
 }
@@ -915,7 +956,7 @@ EST_String EST_String::quote_if_needed(const char quotec) const
 EST_String EST_String::unquote_if_needed(const char quotec) const
 {
 
-  if ((*this)[0] == quotec && (*this)[length()-1] == quotec )
+  if ((*this)(0) == quotec && (*this)(length()-1) == quotec )
     return unquote(quotec);
 
   return *this;
@@ -934,9 +975,16 @@ EST_String EST_String::cat(EST_String s1,
 			  EST_String s2, 
 			  EST_String s3, 
 			  EST_String s4, 
-			  EST_String s5)
+			  EST_String s5,
+			  EST_String s6,
+			  EST_String s7,
+			  EST_String s8,
+			  EST_String s9
+			   )
 {
-  int   len=s1.length()+s2.length()+s3.length()+s4.length()+s5.length();
+  int   len=(s1.length()+s2.length()+s3.length()+s4.length()+s5.length() +
+	     s6.length()+s7.length()+s8.length()+s9.length());
+
   EST_String result;
 
   result.size=len;
@@ -951,6 +999,14 @@ EST_String EST_String::cat(EST_String s1,
     { strncpy((char *)result.memory + p, (const char *)s4, s4.length()); p += s4.length(); }
   if (s5.length())
     { strncpy((char *)result.memory + p, (const char *)s5, s5.length()); p += s5.length(); }
+  if (s6.length())
+    { strncpy((char *)result.memory + p, (const char *)s6, s6.length()); p += s6.length(); }
+  if (s7.length())
+    { strncpy((char *)result.memory + p, (const char *)s7, s7.length()); p += s7.length(); }
+  if (s8.length())
+    { strncpy((char *)result.memory + p, (const char *)s8, s8.length()); p += s8.length(); }
+  if (s9.length())
+    { strncpy((char *)result.memory + p, (const char *)s9, s9.length()); p += s9.length(); }
 
     result.memory(p) = '\0';
 
@@ -981,7 +1037,7 @@ int compare(const EST_String &a, const char *b)
 	return strcmp(a.str(), b);
 }
 
-int fcompare(const EST_String a, const EST_String b, 
+int fcompare(const EST_String &a, const EST_String &b, 
 				const unsigned char *table) 
 {
     if (a.size == 0 && b.size == 0)
@@ -994,6 +1050,20 @@ int fcompare(const EST_String a, const EST_String b,
 	return EST_strcasecmp(a.str(), b.str(), table);
 }
 
+int fcompare(const EST_String &a, const char *b, 
+				const unsigned char *table) 
+{
+    int bsize = (b ? strlen((const char *)b) : 0);
+    if (a.size == 0 && bsize == 0)
+	return 0;
+    else if (a.size == 0)
+	return -1;
+    else if (bsize == 0)
+	return 1;
+    else 
+	return EST_strcasecmp(a.str(), (const char *)b, table);
+}
+
 int operator == (const char *a, const EST_String &b)
 {
     CHECK_STRING_ARG(a);
@@ -1003,7 +1073,7 @@ int operator == (const char *a, const EST_String &b)
     else if (b.size==0)
 	return *a == '\0';
     else 
-	return (*a == b[0]) && strcmp(a, b.str())==0;
+	return (*a == b(0)) && strcmp(a, b.str())==0;
 }
 
 int operator == (const EST_String &a, const EST_String &b)
@@ -1013,5 +1083,5 @@ int operator == (const EST_String &a, const EST_String &b)
     else if (b.size == 0)
 	return 0;
     else 
-	return a.size == b.size && a[0] == b[0] && memcmp(a.str(),b.str(),a.size)==0;
+	return a.size == b.size && a(0) == b(0) && memcmp(a.str(),b.str(),a.size)==0;
 };
