@@ -2,7 +2,7 @@
 #include "regexp.h"
 
 void
-regerror(s)
+hs_regerror(s)
 const char *s;
 {
 #ifdef ERRAVAIL
