@@ -72,13 +72,17 @@ class EST_NgrammarState {
 private:
 
 protected:
-    int p_id; // a 'name'
     EST_DiscreteProbDistribution p_pdf;
-    // to come ?
-    // ... following states
+    int p_id; // a 'name'
 
 public:
-    EST_NgrammarState(){ init(); };
+    EST_NgrammarState() : 
+      
+      p_pdf() 
+
+      { 
+	init(); 
+      };
     EST_NgrammarState(int id,EST_Discrete *d){clear();init(id,d);};
     EST_NgrammarState(int id,const EST_DiscreteProbDistribution &pdf)
               {clear();init(id,pdf);};
@@ -397,10 +401,10 @@ public:
     int states(void) const { return p_num_states;}
     double samples(void) const { return p_num_samples;}
     int order() const { return p_order; }
-    int get_vocab_size() const { return vocab?vocab->size():0; }
+    int get_vocab_length() const { return vocab?vocab->length():0; }
     EST_String get_vocab_word(int i) const;
     int get_vocab_word(const EST_String &s) const;
-    int get_pred_vocab_size() const { return pred_vocab->size(); }
+    int get_pred_vocab_length() const { return pred_vocab->length(); }
     EST_String get_pred_vocab_word(int i) const { return pred_vocab->name(i); }
     int get_pred_vocab_word(const EST_String &s) const 
        { return pred_vocab->name(s); }
@@ -468,6 +472,9 @@ public:
     const EST_String &predict(const EST_IVector &words,double *prob) const
        {int state; return predict(words,prob,&state); }
     
+    int find_state_id(const EST_StrVector &words) const;
+    int find_state_id(const EST_IVector &words) const;
+    int find_next_state_id(int state, int word) const;
     // fast versions for common N
     //const double probability(const EST_String w1);
     //const double probability(const EST_String w1,const EST_String w2);
@@ -483,6 +490,7 @@ public:
     // predict, where words has 'order' elements and the last one is "" or NULL
     const EST_DiscreteProbDistribution &prob_dist(const EST_StrVector &words) const;
     const EST_DiscreteProbDistribution &prob_dist(const EST_IVector &words) const;
+    const EST_DiscreteProbDistribution &prob_dist(int state) const;
     
     bool stats(const EST_String filename,
 	       double &raw_entropy, double &count,
