@@ -37,33 +37,23 @@
 #include <fstream.h>
 #include <math.h>
 #include "EST_types.h"
-#include "EST_stream_aux.h"
-#include "EST_Stream.h"
+#include "ling_class/EST_Stream.h"
+#include "ling_class/EST_stream_aux.h"
 #include "EST_string_aux.h"
 #include "EST_io_aux.h"
-#include "EST_Track.h"
 #include "EST_Option.h"
-//#include "EST_TMatrix.h"
 #include "EST_Token.h"
 
-// temporary commenting out !!!!!  look for the following line :
-// temporary !!!    
+// now in hash_sstreamp_t.cc
+//#if defined(INSTANTIATE_TEMPLATES)
+//#include "../base_class/EST_THash.cc"
+//template class EST_TStringHash<EST_Stream*>;
+//template class EST_THash<EST_String,EST_Stream*>;
+//template class EST_Hash_Pair<EST_String,EST_Stream*>;
+//EST_Stream* EST_THash<EST_String,EST_Stream*>::Dummy_Value = NULL;
+//#endif
 
-int is_in_class(const EST_String &name, const EST_String class_e[], int num);
-int pos_phone(EST_Stream &seg, float x, float shift);
-
-
-void Xshow_links(EST_Stream &a, EST_Stream &b)
-{
-    EST_Stream_Item *a_ptr;
-
-    for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
-    {
-	if (a_ptr->link(b.stream_name())->head())
-	    cout << "Link " << a_ptr->addr() << ": "
-		<< *(a_ptr->link(b.stream_name()));
-    }
-}
+static int is_in_class(const EST_String &name, EST_StrList &s);
 
 void quantize(EST_Stream &a, float q)
 {
@@ -131,108 +121,10 @@ int edit_labels(EST_Stream &a, EST_String sedfile)
     return 0;
 }
 
-// note: tr is not a reference because it may be changed within this
-// function, specifically for re-scaling.
-
-void track_to_label(EST_Track tr, EST_Stream &lab, float thresh)
-{
-    int i;
-    EST_String str[10], tmpstr;
-    EST_Stream_Item tmp_seg;
-    int p_pos = FALSE;
-    int c_pos = FALSE;
-    
-    for (i = 0; i < tr.num_frames(); ++i)
-    {
-	if (tr.a(i) > thresh)
-	    c_pos = TRUE;
-	else 
-	    c_pos = FALSE;
-	
-	if (c_pos == p_pos)
-	{
-	    p_pos = c_pos;
-	    continue;
-	}
-
-	if (c_pos == TRUE)
-	    tmp_seg.set_name("neg");
-	else 
-	    tmp_seg.set_name("pos");
-
-	tmp_seg.set_end(tr.t(i - 1));
-	lab.append(tmp_seg);
-	p_pos = c_pos;
-    }
-    if (c_pos)
-	tmp_seg.set_name("pos");
-    else 
-	tmp_seg.set_name("neg");
-
-    tmp_seg.set_end(tr.t(i - 1));
-    lab.append(tmp_seg);
-}
-
-EST_Track label_to_track(EST_Stream &lab, float shift, float offset, float
-		     range, float req_l, EST_String pad)
-{
-    EST_Track tr;
-    EST_Stream_Item tmp_seg;
-    int i;
-    int n, endn;
-
-// should already be in seconds now !
-//    shift /= 1000.0; // put shift into seconds.
-
-    n = (int)ceil(lab.tail()->end()/shift);
-    endn = (req_l > 0.0) ? (int)(req_l /shift) : n;
-
-//    cout << req_l << endl;
-//    cout << "shift " << shift << endl;
-//    cout << "endn is " << endn << endl;
-//    cout << lab.tail()->end() << " " << shift << endl;
-
-    tr.resize(endn, 1);
-    tr.set_num_frames(endn);
-    tr.fill_time(shift);
-
-    for (i = 0; i < n; ++i)
-    {
-	tr.a(i) = (pos_phone(lab, tr.t(i), shift) * range) + offset;
-	tr.set_value(i);
-    }
-    for (; i < endn; ++i)
-    {
-	tr.a(i) = (pad == "high") ? range + offset : offset;
-	tr.set_value(i);
-    }
-
-    return tr;
-}
-
-EST_Track label_to_track(EST_Stream &lab, EST_Option &al, EST_Option &op)
-{
-    EST_Track tr;
-	
-    float shift = op.present("frame_shift") ? op.fval("frame_shift"): 0.01;
-    float offset = op.present("label_offset")? op.fval("label_offset"):0.0;
-	
-    float range = op.present("label_range") ? op.fval("label_range"): 1.0;
-    float length = al.present("-length") ? al.fval("-length") : -1.0;
-	
-    tr = label_to_track(lab, shift, offset, range, length, 
-			al.val("-pad", 0));
-    tr.amin = 0.0;
-    tr.amax = 3.0;
-
-    return tr;
-}
-
 // make new EST_Stream from start and end points.
-EST_Stream extract(EST_Stream &orig, float start, float end)
+void extract(const EST_Stream &orig, float start, float end, EST_Stream &ex)
 {
     EST_Stream_Item *a;
-    EST_Stream ex;
     EST_Stream_Item tmp;
 
     for (a = orig.head(); a != 0; a = next(a))
@@ -248,22 +140,23 @@ EST_Stream extract(EST_Stream &orig, float start, float end)
 		ex.append(*a);
 	}
 
-    return ex;
 }
 
 void merge_all_label(EST_Stream &seg, const EST_String &labtype)
 {
-    EST_Stream_Item *a_ptr;
+    EST_Stream_Item *a_ptr, *n_ptr;
     (void)labtype;  // unused parameter
 
-    for (a_ptr = seg.head(); a_ptr != seg.tail(); a_ptr = next(a_ptr))
+    for (a_ptr = seg.head(); a_ptr != seg.tail(); a_ptr = n_ptr)
     {
+	n_ptr = next(a_ptr);
 	if (a_ptr->name() == next(a_ptr)->name())
 	    seg.remove(a_ptr);
     }
 }
 
-void change_label(EST_Stream &seg, const EST_String &oname, const EST_String &nname)
+void change_label(EST_Stream &seg, const EST_String &oname, 
+		  const EST_String &nname)
 {
     EST_Stream_Item *a_ptr;
 
@@ -272,7 +165,19 @@ void change_label(EST_Stream &seg, const EST_String &oname, const EST_String &nn
 	    a_ptr->set_name(nname);
 }
 
-int is_in_class(const EST_String &name, EST_StrList &s)
+void change_label(EST_Stream &seg, const EST_StrList &oname, 
+		  const EST_String &nname)
+{
+    EST_Stream_Item *a_ptr;
+    EST_Litem *p;
+
+    for (a_ptr = seg.head(); a_ptr != 0; a_ptr = next(a_ptr))
+	for (p = oname.head(); p ; p = next(p))
+	    if (a_ptr->name() == oname(p))
+		a_ptr->set_name(nname);
+}
+
+static int is_in_class(const EST_String &name, EST_StrList &s)
 {
     EST_TBI *p;
 
@@ -295,8 +200,6 @@ int check_vocab(EST_Stream &a, EST_StrList &vocab)
     return 0;
 }
 
-
-
 void convert_to_broad_class(EST_Stream &seg, const EST_String &class_type, 
 			   EST_Option &options)
 {
@@ -311,12 +214,12 @@ void convert_to_broad_class(EST_Stream &seg, const EST_String &class_type,
     ts.open_string(bc_list);
     while (!ts.eof())
         pos_list.append(ts.get().string());
-    
+
     convert_to_broad(seg, pos_list);
 } 
 
 void convert_to_broad(EST_Stream &seg, EST_StrList &pos_list, 
-		      EST_String broad_name)
+		      EST_String broad_name, int polarity)
 {
     EST_Stream_Item *a_ptr;
     if (broad_name == "")
@@ -324,9 +227,9 @@ void convert_to_broad(EST_Stream &seg, EST_StrList &pos_list,
 
     for (a_ptr = seg.head(); a_ptr != 0; a_ptr = next(a_ptr))
 	if (is_in_class(a_ptr->name(), pos_list))
-	    a_ptr->set_feature(broad_name, 1);
+	    a_ptr->f.set(broad_name, (polarity) ? 1 : 0);
 	else
-	    a_ptr->set_feature(broad_name, 0);
+	    a_ptr->f.set(broad_name, (polarity) ? 0 : 1);
 } 
 
 void label_map(EST_Stream &seg, EST_Option &map)
@@ -356,21 +259,6 @@ void label_map(EST_Stream &seg, EST_Option &map)
     }
 } 
 
-int pos_phone(EST_Stream &seg, float x, float shift)
-{
-    // returns true if x is in a positive segment. The decision is
-    // slightly biased towards positive inclusion when x is near
-    // a boundary.
-    EST_Stream_Item *a_ptr;
-    
-    for (a_ptr = seg.head(); a_ptr != 0; a_ptr = next(a_ptr))
-	if (a_ptr->feature("pos") == 1)
-	    if ((x < (a_ptr->end() + (shift / 2.0))) &&
-		(x > (a_ptr->start() -  (shift / 2.0))))
-		return 1;
-    return 0;
-}
-
 void shift_label(EST_Stream &seg, float shift)
 {
     //shift every end time by adding x seconds.
@@ -431,7 +319,7 @@ void StreamList_select(EST_StreamList &mlf, EST_StrList filenames, bool
 
 // look for a single file called "filename" and make a EST_Stream out of
 // this
-EST_Stream StreamList_extract(EST_StreamList &mlf, EST_String filename, bool base)
+EST_Stream StreamList_extract(EST_StreamList &mlf, const EST_String &filename, bool base)
 {
     EST_TBI *p;
     EST_String test, ref;
@@ -498,31 +386,11 @@ EST_Stream StreamList_combine(EST_StreamList &mlf, EST_Stream &key)
 	    t.set_name(s->name());
 	    t.set_end(s->end() + start);
 	    all.append(t);
-//	    cout << "appended t " << t << endl;
 	}
     }
     return all;
 }
 
-int find_filename(EST_StrList &f, EST_String &filename)
-{
-    EST_TBI *p;
-    EST_String t;
-
-    for (p = f.head(); p; p = next(p))
-	if (f(p).contains("file"))
-	{
-	    filename = f(p);
-	    filename = filename.after("file");
-	    filename.gsub(RXwhite, "");
-	    return 0;
-	}
-
-    cerr << "No file definition found in field\n";
-    return -1;
-}
-
-
 int stream_divide(EST_StreamList &slist, EST_Stream &lab, EST_Stream &keylab,
 		  EST_StrList &blank, EST_String ext)
 { // divides a single stream into multiple chunks according to the
@@ -550,7 +418,7 @@ int stream_divide(EST_StreamList &slist, EST_Stream &lab, EST_Stream &keylab,
 	if (k->end() > lab.head()->end())
 	    break;
 
-    find_filename(k->fields, filename);
+    filename = k->f("file");
     a.set_name(filename + ext);
     kstart = 0.0;
     
@@ -589,7 +457,7 @@ int stream_divide(EST_StreamList &slist, EST_Stream &lab, EST_Stream &keylab,
 	    k = next(k);
 	    kstart = k->start();
 	    a.clear();
-	    find_filename(k->fields, filename);
+	    filename = k->f("file");
 	    a.set_name(filename + ext);
 	}
 	else
@@ -604,49 +472,6 @@ int stream_divide(EST_StreamList &slist, EST_Stream &lab, EST_Stream &keylab,
     return 0;
 }
 
-/*
-   
-   if (n->end() > k->end())
-   {
-   cout << "1 " << (s->end() - k->end()) << endl;
-   cout << "2 " << (k->end() - s->start()) << endl;
-   if ((n->end() - k->end()) < (k->end() - n->start()))
-   {
-   t.set_name(s->name());
-   t.set_end(s->end() - kstart);
-   a.append(t);
-   s = next(s);
-   t.set_name(s->name());
-   t.set_end(s->end() - kstart);
-   a.append(t);
-   }
-   else
-   {
-   t.set_name(s->name());
-   t.set_end(k->end() - kstart);
-   a.append(t);
-   }
-   
-   mlf.append(a);
-   kstart = s->end();
-   k = next(k);
-   a.clear();
-   find_filename(k->fields, filename);
-   a.set_name(filename + ext);
-   }
-   
-   else
-   {
-   t.set_name(s->name());
-   t.set_end(s->end() - kstart);
-   a.append(t);
-   }
-   }
-   mlf.append(a);
-   
-   return 0;
-   }
-   */
 int stream_divide2(EST_StreamList &mlf, EST_Stream &lab, EST_Stream &keylab,
 		   EST_String ext)
 {
@@ -690,7 +515,7 @@ int stream_divide2(EST_StreamList &mlf, EST_Stream &lab, EST_Stream &keylab,
     return 0;
 }
 
-EST_TBI *StreamList_ptr_extract(EST_StreamList &mlf, EST_String filename, bool base)
+EST_TBI *StreamList_ptr_extract(EST_StreamList &mlf, const EST_String &filename, bool base)
 {
     EST_TBI *p;
     EST_String test, ref;
@@ -728,7 +553,7 @@ void stream_convert(EST_Stream &lab, EST_Option &al, EST_Option &op)
 	if (!al.present("-end"))
 	    cerr << "-start option must be used with -end option\n";
 	else 
-	    lab = extract(lab, al.fval("-start"), al.fval("-end"));
+	    extract(lab, al.fval("-start"), al.fval("-end"), lab);
     }
     
     if (al.present("-class"))
@@ -758,11 +583,50 @@ void print_stream_features(EST_Stream &stream)
     for (s = stream.head(); s; s = next(s))
     {
 	cout << s->name() << "\t:";
-	for (p = s->features.list.head(); p; p = next(p))
-	    cout << s->features.list(p).k << " " 
-		<< s->features.list(p).v << "; ";
+	for (p = s->f.head(); p; p = next(p))
+	    cout << s->f.fname(p) << " " 
+		<< s->f.fval(p) << "; ";
 	cout << endl;
     }
     
 }
 
+
+void build_StreamList_hash_table(const EST_StreamList &mlf,
+				 EST_TStringHash<EST_Stream*> &hash_table, 
+				 const bool base)
+{
+
+    EST_Litem *p;
+    if (base)
+	for (p = mlf.head(); p; p = next(p))
+	    hash_table.add_item(basename(mlf(p).name(), "*"),
+				&(((EST_TItem<EST_Stream> *)p)->val));
+    
+    else 
+	for (p = mlf.head(); p; p = next(p))
+	    hash_table.add_item(mlf(p).name(),
+				&(((EST_TItem<EST_Stream> *)p)->val));
+
+}
+
+EST_Stream hashed_StreamList_extract(const EST_TStringHash<EST_Stream*> &hash_table,
+				      const EST_String &filename, bool base)
+{
+    EST_Stream *d;
+    EST_String fname = filename;
+    int found;
+
+    if (base)
+	fname=basename(filename, "*");
+
+    d=hash_table.val(fname,found);
+
+    if(found)
+	return *d;
+
+    cerr << "No match for file " << filename << " found in mlf\n";
+    EST_Stream dd;
+    return dd;
+}
+
