@@ -32,16 +32,17 @@
 /*                  Stream class auxiliary routines                      */
 /*                                                                       */
 /*=======================================================================*/
+
 #include <iostream.h>
 #include <fstream.h>
 #include <math.h>
 #include "EST_types.h"
 #include "EST_FMatrix.h"
-#include "EST_Stream.h"
+#include "ling_class/EST_Stream.h"
 #include "EST_Token.h"
 #include "EST_string_aux.h"
-#include "EST_stream_aux.h"
-#include "EST_stream_compare.h"
+#include "ling_class/EST_stream_aux.h"
+#include "ling_class/EST_stream_compare.h"
 #include "EST_io_aux.h"
 
 
@@ -50,61 +51,27 @@ int close_enough(EST_Stream_Item &a, EST_Stream_Item &b)
     return ((b.start() < a.end()) && (a.start() < b.end()));
 }
 
-void show_links(EST_Stream &a, EST_Stream &b)
-{
-    EST_Stream_Item *a_ptr;
-
-    for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
-    {
-	if (a_ptr->link(b.stream_name())->head())
-	    cout << "Link " << a_ptr->addr() << ": " 
-		<< *(a_ptr->link(b.stream_name()));
-    }
-}
-
-// WATCH - this uses what should be private access to Keyval class.
-void monotonic_match(EST_II_KVL &a, EST_II_KVL &b)
-{
-    EST_TBI *ptr;
-
-    for (ptr = a.list.head(); ptr != 0; ptr= next(ptr))
-    {
-	if (a.valp(ptr) == -1)
-	    continue;
-	if (b.val(a.valp(ptr)) == a.key(ptr))
-//	    cout << "ok\n";
-	    continue;
-	else
-	    a.change_item(ptr, -1);
-    }
-    for (ptr = b.list.head(); ptr != 0; ptr= next(ptr))
-    {
-	if (b.valp(ptr) == -1)
-	    continue;
-	if (a.val(b.valp(ptr)) == b.key(ptr))
-//	    cout << "ok\n";
-	    continue;
-	else
-	    a.change_item(ptr, -1);
-    }
-}
-
 void function_match(EST_II_KVL &u, EST_Stream &a, EST_Stream &b)
 {
+    (void)u;
+    (void)a;
+    (void)b;
+#if 0
     EST_Stream_Item *a_ptr;
     EST_TBI *i_ptr;
     int i;
 
     for (i = 0, a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr), ++i)
     {
-	if (a_ptr->feature("pos")==1)
+	if (a_ptr->f("pos")==1)
 	{
 	    u.add_item(a_ptr->addr(), -1);
 	    for (i_ptr = a_ptr->link(b.stream_name())->head(); i_ptr != 0; 
 		 i_ptr = next(i_ptr))
-		u.change_item(a_ptr->addr(), a_ptr->link(b.stream_name())->item(i_ptr));
+		u.change_val(a_ptr->addr(), a_ptr->link(b.stream_name())->item(i_ptr));
 	}
     }
+#endif
 }
 
 void relation_match(EST_Stream &a, EST_Stream &b)
@@ -112,14 +79,16 @@ void relation_match(EST_Stream &a, EST_Stream &b)
     EST_Stream_Item *a_ptr, *b_ptr;
 
     for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
-	if (a_ptr->feature("pos")==1)
+	if (a_ptr->f("pos")==1)
 	    for (b_ptr = b.head(); b_ptr != 0; b_ptr = next(b_ptr))
 		{
-		    if ((b_ptr->feature("pos")==1)
+		    if ((b_ptr->f("pos")==1)
 			&&(close_enough(*a_ptr, *b_ptr)))
 		    {
 //			cout << "linked\n";
+#if 0
 			link(*a_ptr, *b_ptr);
+#endif
 		    }
 		}
 
@@ -128,54 +97,12 @@ void relation_match(EST_Stream &a, EST_Stream &b)
 //		    link(*a_ptr, *b_ptr);
 }
 
-void compare_labels(EST_Stream &reflab, EST_Stream &testlab)
-{
-    EST_II_KVL uref, utest;
-
-    relation_match(reflab, testlab); // many-to-many mapping
-
-    cout << "Ref\n" << reflab;
-    cout << "Test\n" << testlab;
-
-    function_match(uref, reflab, testlab); // one-to-many mapping
-    function_match(utest, testlab, reflab); // one-to-many mapping
-
-    cout << "Ref\n" << reflab;
-    cout << "Test\n" << testlab;
-    cout << "Keyval REF\n" << uref;
-    cout << "Keyval TEST\n" << utest;
-
-//    cout << "Keyval REF\n" << uref;
-//    cout << "Keyval TEST\n" << utest;
-
-    monotonic_match(uref, utest); // one-to-one mapping
-
-    reassign_links(reflab, uref, testlab.stream_name());
-    reassign_links(testlab, utest, reflab.stream_name());
-    cout << "Keyval REF\n" << uref;
-    cout << "Keyval TEST\n" << utest;
-
-// temporary !!!    
-
-    cout.setf(ios::left,ios::adjustfield);
-
-    cout << "Total: ";
-    cout.width(10);
-    cout << uref.length();
-    cout << "Deletions: ";
-    cout.width(10);
-    cout << insdel(uref);
-    cout << "Insertions: "; 
-    cout.width(10);
-    cout<< insdel(utest) << endl;
-}
-
 EST_Stream_Item *nthpos(EST_Stream &a, int n)
 {
     EST_Stream_Item *a_ptr;
     int i = 0;
     for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
-	if (a_ptr->feature("pos") == 1)
+	if (a_ptr->f("pos") == 1)
 	{
 	    if (n == i)
 		return a_ptr;
@@ -207,78 +134,6 @@ float label_distance2(EST_Stream_Item &ref, EST_Stream_Item &test)
     return (start + end) / ref.dur();
 }
 
-int lowest_pos(EST_FMatrix &m, int j)
-{
-    float val = 1000.0;
-    int i, pos=0;
-
-    for (i = 0; i < m.num_rows(); ++i)
-	if ((m(i, j) > -0.01) && (m(i, j) < val))
-	{
-	    val = m(i, j);
-	    pos = i;
-	}
-
-    return pos;
-}
-
-void minimise_matrix_by_column(EST_FMatrix &m)
-{
-    float val = 1000.0;
-    int i;
-    for (int j = 0; j < m.num_columns(); ++j)
-    {
-	val = 1000.0;
-	for (i = 0; i < m.num_rows(); ++i)
-	    if (m(i, j) < val)
-		val = m(i, j);
-	for (i = 0; i < m.num_rows(); ++i)
-	    if (m(i, j) > val)
-		m(i, j) = -1.0;
-    }
-}
-
-void minimise_matrix_by_row(EST_FMatrix &m)
-{
-    float val;
-    int i, j;
-
-    for (i = 0; i < m.num_rows(); ++i)
-    {
-	val = 1000.0;
-	for (j = 0; j < m.num_columns(); ++j)
-	    if ((m(i, j) < val) && (m(i, j) > -0.01))
-		val = m(i, j);
-	for (j = 0; j < m.num_columns(); ++j)
-	    if (m(i, j) > val)
-		m(i, j) = -1.0;
-    }
-}
-
-void matrix_ceiling(EST_FMatrix &m, float max)
-{
-    int i, j;
-
-    for (i = 0; i < m.num_rows(); ++i)
-	for (j = 0; j < m.num_columns(); ++j)
-	    if (m(i, j) > max)
-		m(i, j) = -1.0;
-
-}
-
-int matrix_insertions(EST_FMatrix &m)
-{
-    int i, j;
-    int n = 0;
-
-    for (i = 0; i < m.num_rows(); ++i)
-	for (j = 0; j < m.num_columns(); ++j)
-	    if (m(i, j) > -1.0)
-		++n;
-
-    return (m.num_rows() - n);
-}
-
 int major_matrix_insertions(EST_FMatrix &m, EST_Stream &ref_lab)
 {
     int i, j;
@@ -288,8 +143,8 @@ int major_matrix_insertions(EST_FMatrix &m, EST_Stream &ref_lab)
     for (i = 0; i < m.num_rows(); ++i)
     {
 	s = nthpos(ref_lab, i);
-//	cout << s->name() << ": f:" << s->feature("minor")<< endl;
-	if (s->feature("minor") == 1)
+//	cout << s->name() << ": f:" << s->f("minor")<< endl;
+	if (s->f("minor") == 1)
 	    ++n;
 	else
 	    for (j = 0; j < m.num_columns(); ++j)
@@ -299,19 +154,6 @@ int major_matrix_insertions(EST_FMatrix &m, EST_Stream &ref_lab)
     return (m.num_rows() - n);
 }
 
-int matrix_deletions(EST_FMatrix &m)
-{
-    int i, j;
-    int n = 0;
-
-    for (j = 0; j < m.num_columns(); ++j)
-	for (i = 0; i < m.num_rows(); ++i)
-	    if (m(i, j) > -1.0)
-		++n;
-
-    return (m.num_columns() - n);
-}
-
 int major_matrix_deletions(EST_FMatrix &m, EST_Stream &ref_lab)
 {
     int i, j;
@@ -321,8 +163,8 @@ int major_matrix_deletions(EST_FMatrix &m, EST_Stream &ref_lab)
     for (j = 0; j < m.num_columns(); ++j)
     {
 	s = nthpos(ref_lab, j);
-//	cout << s->name() << ": f:" << s->feature("minor")<< endl;
-	if (s->feature("minor") == 1)
+//	cout << s->name() << ": f:" << s->f("minor")<< endl;
+	if (s->f("minor") == 1)
 	    ++n;
 	else
 	    for (i = 0; i < m.num_rows(); ++i)
@@ -333,38 +175,29 @@ int major_matrix_deletions(EST_FMatrix &m, EST_Stream &ref_lab)
     return (m.num_columns() - n);
 }
 
-int lowest_pos(float *m, int n)
-{
-    float val = 1000.0;
-    int i, pos=0;
-
-    for (i = 0; i < n; ++i)
-	if (m[i] < val)
-	{
-	    val = m[i];
-	    pos = i;
-	}
-
-    return pos;
-}
-
 void threshold_labels(EST_Stream &reflab, float t)
 {
+    (void)reflab;
+    (void)t;
+#if 0
     EST_Stream_Item *r_ptr;
     float score=0.0;
     int a;
 
     for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
-	if (r_ptr->feature("pos")==1)
+	if (r_ptr->f("pos")==1)
 	{
 	    // REORG - temp comment
 //	    score = atof(r_ptr->fields());
 	    cout << "score is" << score << endl;
+
 	    a = r_ptr->rlink("blank").first();
+
 	    cout << "score is" << score << " address: " << a << endl;
 	    if (score > t)
 		cout << "delete\n";
 	}
+#endif
 }
 
 /* Check through relations of each ref label, and make there aren't
@@ -378,6 +211,8 @@ test labels.
 
 void multiple_labels(EST_Stream &reflab)
 {
+    (void)reflab;
+#if 0
     EST_Stream_Item *r_ptr, *s_ptr, *t_ptr;;
     EST_TBI *p;
     int a, pos, i;
@@ -386,7 +221,7 @@ void multiple_labels(EST_Stream &reflab)
     score = new float[reflab.length()];
 
     for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
-	if (r_ptr->feature("pos")==1)
+	if (r_ptr->f("pos")==1)
 	{
 	    la.clear(); // clear list and add address of current ref label
 	    la.append(r_ptr->addr());
@@ -396,7 +231,7 @@ void multiple_labels(EST_Stream &reflab)
 	    // check remainer of ref labels and add any that have same
 	    // relations address as r_ptr.
 	    for (s_ptr = next(r_ptr); s_ptr != 0; s_ptr = next(s_ptr))
-		if (s_ptr->feature("pos")==1)
+		if (s_ptr->f("pos")==1)
 		    if (s_ptr->rlink("test").first() == a)
 			la.append(s_ptr->addr());
 
@@ -423,6 +258,8 @@ void multiple_labels(EST_Stream &reflab)
 		    }
 	    }
 	}
+#endif
+
 }
 
 /* Compare 2 sets of labels by matrix method. This involves making a M
@@ -447,11 +284,11 @@ EST_FMatrix matrix_compare(EST_Stream &reflab, EST_Stream &testlab, int method,
 
     // calculate size of matrix, based on *significant* labels
     for (r_ptr = testlab.head(); r_ptr != 0; r_ptr = next(r_ptr))
-	if (r_ptr->feature("pos")==1)
+	if (r_ptr->f("pos")==1)
 	    ++num_test;
 
     for (r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
-	if (r_ptr->feature("pos")==1)
+	if (r_ptr->f("pos")==1)
 	    ++num_ref;
 
     EST_FMatrix m(num_test, num_ref);
@@ -463,10 +300,10 @@ EST_FMatrix matrix_compare(EST_Stream &reflab, EST_Stream &testlab, int method,
     // reference is columns, test is rows
 
     for (i = 0, t_ptr = testlab.head(); t_ptr != 0; t_ptr = next(t_ptr))
-	if (t_ptr->feature("pos")==1)
+	if (t_ptr->f("pos")==1)
 	{
 	    for (j = 0, r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
-		if (r_ptr->feature("pos")==1)
+		if (r_ptr->f("pos")==1)
 		{
 		    if (method == 1)
 			m(i, j) = label_distance1(*r_ptr, *t_ptr);
@@ -488,7 +325,7 @@ EST_FMatrix matrix_compare(EST_Stream &reflab, EST_Stream &testlab, int method,
     // for each ref label, find closest matching test label.
     for (j = 0, r_ptr = reflab.head(); r_ptr != 0; r_ptr = next(r_ptr))
     {
-	if (r_ptr->feature("pos")==1)
+	if (r_ptr->f("pos")==1)
 	{
 	    pos = lowest_pos(m, j);
 	    // REORG - temp comment
@@ -578,88 +415,66 @@ void error_location(EST_Stream &e, EST_FMatrix &m, int ref)
     if (ref)
     {
 	for (i = 0, s = e.head(); s; s = next(s))
-	    if ((int)s->feature("pos"))
+	    if ((int)s->f("pos"))
 	    {
 		if (column_hit(m, i) >= 0)
-		    s->set_feature("hit", 1);
+		    s->f.set("hit", 1);
 		else
-		    s->set_feature("hit", 0);
+		    s->f.set("hit", 0);
 		++i;
 	    }
     }
     else
 	for (i = 0, s = e.head(); s; s = next(s))
-	    if ((int)s->feature("pos"))
+	    if ((int)s->f("pos"))
 	    {
 		if (row_hit(m, i) >= 0)
-		    s->set_feature("hit", 1);
+		    s->f.set("hit", 1);
 		else
-		    s->set_feature("hit", 0);
+		    s->f.set("hit", 0);
 		++i;
 	    }
 }
 
-int insdel(EST_II_KVL &a)
-{
-    int n = 0;
-    EST_TBI *ptr;
-    
-    for (ptr = a.list.head(); ptr != 0; ptr= next(ptr))
-	if (a.valp(ptr) == -1)
-	    ++n;
-    return n;
-}
-
-int compare_labels(EST_Utterance &ref, EST_Utterance &test, EST_String name,
-		   EST_II_KVL &uref, EST_II_KVL &utest)
-{
-    // many-to-many mapping
-    relation_match(ref.stream(name), test.stream(name)); 
-    
-    // one-to-many mapping
-    function_match(uref, ref.stream(name), test.stream(name));
-    function_match(utest, test.stream(name), ref.stream(name)); 
-    
-    monotonic_match(uref, utest); // one-to-one mapping
-    
-    // temporary !!!    
-    //    reassign_links(ref.stream(name), uref, name);
-    //    reassign_links(test.stream(name), utest, name);
-    
-    //    cout << "Keyval REF\n" << uref;
-    //    cout << "Keyval TEST\n" << utest;
-    
-    return 0;
-}
-
 void reassign_links(EST_Stream &a, EST_Stream &b, EST_II_KVL &ua, EST_II_KVL &ub)
 {
+
+  (void)a;
+  (void)b;
+  (void)ua;
+  (void)ub;
+#if 0    
     EST_Stream_Item *a_ptr, *b_ptr;
-    
     for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
     {
 	a_ptr->link(b.stream_name())->clear();
-	if ((a_ptr->feature("pos")==1) && (ua.val(a_ptr->addr()) != -1))
+	if ((a_ptr->f("pos")==1) && (ua.val(a_ptr->addr()) != -1))
 	    a_ptr->make_link(b.stream_name(), ua.val(a_ptr->addr()));
     }
     for (b_ptr = b.head(); b_ptr != 0; b_ptr = next(b_ptr))
     {
 	b_ptr->link(a.stream_name())->clear();
-	if ((b_ptr->feature("pos")==1) && (ub.val(b_ptr->addr()) != -1))
+	if ((b_ptr->f("pos")==1) && (ub.val(b_ptr->addr()) != -1))
 	    b_ptr->make_link(a.stream_name(), ub.val(b_ptr->addr()));
     }
+#endif
 }
 
 void reassign_links(EST_Stream &a, EST_II_KVL &u, EST_String stream_type)
 {
+    (void)a;
+    (void)u;
+    (void)stream_type;
+#if 0    
     EST_Stream_Item *a_ptr;
-    
+
     for (a_ptr = a.head(); a_ptr != 0; a_ptr = next(a_ptr))
     {
 	a_ptr->link(stream_type)->clear();
-	if ((a_ptr->feature("pos")==1) && (u.val(a_ptr->addr()) != -1))
+	if ((a_ptr->f("pos")==1) && (u.val(a_ptr->addr()) != -1))
 	    a_ptr->make_link(stream_type, u.val(a_ptr->addr()));
     }
+#endif
 }
 
 
@@ -693,109 +508,6 @@ int commutate(EST_Stream_Item *a_ptr, EST_II_KVL &f1, EST_II_KVL &f2,
 //	lr(e) == f1(lt(f2(e)))
 // 	Then ei has been recognised fully.
 
-void test_labels(EST_Utterance &ref, EST_Utterance &test, EST_Option &op)
-{
-    EST_II_KVL f2, inv_f2, inv_f1, f1, lref, ltest;
-    
-    compare_labels(ref, test, "Event", f2, inv_f2);
-    compare_labels(ref, test, "Syllable", f1, inv_f1);
-    
-    if (op.present("print_syllable") && op.present("print_map"))
-    {
-	cout << "Syllable mapping from ref to test\n" << f1;
-	cout << "Syllable mapping from test to ref\n" << inv_f1;
-    }
-    if (op.present("print_event") && op.present("print_map"))
-    {
-	cout << "Event mapping from ref to test\n" << f2;
-	cout << "Event mapping from test to ref\n" << inv_f2;
-    }
-    
-    if (op.present("print_syllable") && op.present("print_ins"))
-	cout << "Syllable_insertions: " << insdel(inv_f1) << endl;
-    if (op.present("print_syllable") && op.present("print_del"))
-	cout << "Syllable_deletions: " << insdel(f1) << endl;
-    
-    if (op.present("print_event") && op.present("print_ins"))
-	cout << "Event_insertions: " << insdel(inv_f2) << endl;
-    if (op.present("print_event")  && op.present("print_del"))
-	cout << "Event_deletions: " << insdel(f2) << endl;
-    
-    //    cout << "Ref\n" <<  ref.stream("Event") << ref.stream("Syllable");
-    //    cout << "Test\n" << test.stream("Event") << test.stream("Syllable");
-    
-    function_match(lref, ref.stream("Event"), ref.stream("Syllable"));
-    function_match(ltest, test.stream("Event"), test.stream("Syllable"));
-    
-    if (op.present("print_functions"))
-    {
-	cout << "Lref\n" << lref;
-	cout << "Ltest\n" << ltest;
-	cout << "f1\n" << f1;
-	cout << "f2\n" << f2;
-    }
-    
-    EST_Stream_Item *a_ptr;
-    int correct, n_ev, n_syl;
-    
-    correct = n_ev = n_syl = 0;
-    for (a_ptr = ref.stream("Event").head(); a_ptr != 0; a_ptr = next(a_ptr))
-	if (a_ptr->feature("pos")==1)
-	{
-	    ++n_ev;
-	    if (lref.val(a_ptr->addr())
-		== commutate(a_ptr, f1, f2, lref, ltest))
-		++correct;
-	}
-    for (a_ptr = ref.stream("Syllable").head();a_ptr != 0; a_ptr = next(a_ptr))
-	if (a_ptr->feature("pos")==1)
-	    ++n_syl;
-    
-    if (op.present("print_syllable") && op.present("print_total"))
-	cout << "Number_of_Syllables: " << n_syl << endl;
-    if (op.present("print_event") && op.present("print_total"))
-	cout << "Number_of_Events: " << n_ev << endl;
-    
-    if (op.present("print_link"))
-	cout << "Correct_links: " << correct <<endl;
-    
-    if (op.present("print_derivation"))
-    {
-	for (a_ptr = ref.stream("Event").head();a_ptr!= 0; a_ptr = next(a_ptr))
-	{
-	    if (a_ptr->feature("pos")==1)
-	    {
-		cout << "Lr(ei): " << lref.val(a_ptr->addr()) << endl;
-		cout << "f2(ei): " << f2.val(a_ptr->addr()) << endl;
-		cout << "Lt(f2(ei)): " << ltest.val(f2.val(a_ptr->addr()))
-		    << endl;
-		cout << "f1(Lt(f2(ei))): " 
-		    << f1.val(ltest.val(f2.val(a_ptr->addr()))) << endl;
-	    }
-	    cout << "Event " << *a_ptr;
-	    if ( lref.val(a_ptr->addr())
-		== f1.val(ltest.val(f2.val(a_ptr->addr()))))
-		cout  << " is correct\n";
-	    else
-		cout  << " is incorrect\n";
-	}
-    }
-}
-
-void print_i_d_scores(EST_FMatrix &m)
-{
-    cout.setf(ios::left,ios::adjustfield);
-    cout << "Total: ";
-    cout.width(10);
-    cout << m.num_columns();
-    cout << "Deletions: ";
-    cout.width(10);
-    cout << matrix_deletions(m);
-    cout << "Insertions: "; 
-    cout.width(10);
-    cout<< matrix_insertions(m) << endl;
-}
-
 void print_matrix_scores(EST_Stream &ref, EST_Stream &test, EST_FMatrix &a)
 {
     int i, j;
@@ -804,7 +516,7 @@ void print_matrix_scores(EST_Stream &ref, EST_Stream &test, EST_FMatrix &a)
     cout << "      ";
     for (r_ptr = ref.head(); r_ptr != 0; r_ptr = next(r_ptr))
     {	
-	if (r_ptr->feature("pos")==1)
+	if (r_ptr->f("pos")==1)
 	{
 	    //	cout.width(5);
 	    //	cout.setf(ios::right);
@@ -818,7 +530,7 @@ void print_matrix_scores(EST_Stream &ref, EST_Stream &test, EST_FMatrix &a)
     
     for (t_ptr = test.head(), i = 0; i < a.num_rows(); t_ptr = next(t_ptr))
     {
-	if (t_ptr->feature("pos")==1)
+	if (t_ptr->f("pos")==1)
 	{
 	    cout << t_ptr->name() << " ";
 	    for (j = 0; j < a.num_columns(); ++j)
@@ -835,51 +547,12 @@ void print_matrix_scores(EST_Stream &ref, EST_Stream &test, EST_FMatrix &a)
     }
 }
 
-int row_hit(EST_FMatrix &m, int r)
-{
-    int i;
-    for (i = 0; i < m.num_columns(); ++i)
-	if (m(r, i) > 0.0)
-	    return i;
-    
-    return -1;
-}
-
-// return the row index of the first positive entry in column c
-int column_hit(EST_FMatrix &m, int c)
-{
-    int i;
-    for (i = 0; i < m.num_rows(); ++i)
-	if (m(i, c) > 0.0)
-	    return i;
-    
-    return -1;
-}
-
-int num_b_insertions(EST_FMatrix &m, int last, int current)
-{
-    int c1, c2;
-    c1 = column_hit(m, last);
-    c2 = column_hit(m, current);
-    
-    return c2 - c1 -1;
-}
-
-int num_b_deletions(EST_FMatrix &m, int last, int current)
-{
-    int c1, c2;
-    c1 = row_hit(m, last);
-    c2 = row_hit(m, current);
-    
-    return c2 - c1 -1;
-}
-
 void print_s_trans(EST_Stream &a, int width)
 {
     for (int i = 0; i < a.length(); ++i)
     {
-	//	cout <<	(int)a.nth(i)->feature("pos") << " XX " << a.nth(i)->name() << endl;
-	if ((a.nth(i)->feature("pos")==1) || (a.nth(i)->name() == "   "))
+	//	cout <<	(int)a.nth(i)->f("pos") << " XX " << a.nth(i)->name() << endl;
+	if ((a.nth(i)->f("pos")==1) || (a.nth(i)->name() == "   "))
 	{
 	    //	    cout.setf(ios::fixed, ios::floatfield);
 	    cout.width(width);
@@ -895,13 +568,13 @@ void make_hit_and_miss(EST_Stream &a)
     
     for (s = a.head(); s; s = next(s))
     {
-	if (s->feature("pos") == 0)
+	if (s->f("pos") == 0)
 	    s->set_name(".");
-	else if (s->feature("hit") == 1)
+	else if (s->f("hit") == 1)
 	    s->set_name("HIT");
 	else
 	    s->set_name("MISS");
-	s->features.clear();
+	s->f.clear();
     }
 }
 
@@ -910,7 +583,7 @@ void pos_only(EST_Stream &a)
     EST_Stream_Item *a_ptr;
     
     for (a_ptr = a.head(); a_ptr; )
-	if (a_ptr->feature("pos")==0)
+	if (a_ptr->f("pos")==0)
 	{
 	    a_ptr = a.remove(a_ptr);
 	    if (a_ptr == 0)		// check to see if we are at head of list
@@ -932,8 +605,8 @@ void print_aligned_trans(EST_Stream &ref, EST_Stream &test, EST_FMatrix &m)
     
     blank.set_name("   ");
     
-    pos.set_feature("pos", 1);
-    blank.set_feature("pos", 0);
+    pos.f.set("pos", 1);
+    blank.f.set("pos", 0);
     
     pos_only(test);
     pos_only(ref);
@@ -1056,8 +729,8 @@ void print_results(EST_Stream &ref, EST_Stream &test, EST_FMatrix &m, int tot,
 	make_hit_and_miss(ref);
 	error_location(test, m, 0);
 	make_hit_and_miss(test);
-	ref.save("ref.error", "esps", 3);
-	test.save("test.error", "esps", 3);
+	ref.save("ref.error", "esps", "features");
+	test.save("test.error", "esps", "features");
     }
     else if (v == 7)
     {
@@ -1065,7 +738,7 @@ void print_results(EST_Stream &ref, EST_Stream &test, EST_FMatrix &m, int tot,
 	make_hit_and_miss(ref);
 	error_location(test, m, 0);
 	make_hit_and_miss(test);
-	ref.save("ref.error", "esps", 3);
-	test.save("test.error", "esps", 3);
+	ref.save("ref.error", "esps", "features");
+	test.save("test.error", "esps", "features");
     }
 }
