@@ -38,59 +38,19 @@
 #include <fstream.h>
 #include "EST_unix.h"
 #include "EST_types.h"
-#include "EST_Stream.h"
-#include "stream_io.h"
+#include "ling_class/EST_Stream.h"
 #include "EST_string_aux.h"
 #include "EST_cutils.h"
 #include "EST_TList.h"
 #include "EST_Option.h"
+#include "stream_io.h"
 
 #define DEF_SAMPLE_RATE 16000
 #define HTK_UNITS_PER_SECOND 10000000
 
 EST_read_status read_label_portion(EST_TokenStream &ts, EST_Stream &s, int sample);
 
-const int EST_Val::to_int(void) const
-{
-    // coerce this to an int
-    if (t==val_float)
-	return (int)v.fval;
-    else if (t==val_string)
-	return atoi(sval);
-    else
-	return v.ival;  // just for completeness
-}
-
-const float EST_Val::to_flt(void) const
-{
-    // coerce this to a float
-    if (t==val_int)
-	return (float)v.ival;
-    else if (t==val_string)
-	return atof(sval);
-    else
-	return v.fval;  // just for completeness
-}
-
-const EST_String &EST_Val::to_str(void) const
-{
-    // coerce this to and save it for later
-    // This requires the following casting, so we can still tell the
-    // compiler this is a const function.  If this was properly declared
-    // non-const vast amounts of the rest of this would also have to be
-    // non-const.  So we do one nasty bit here for uniformity elsewhere.
-    // Not saving the result is also a posibility but probably too
-    // inefficient (maybe not with rjc's string class)
-    EST_String *n = (EST_String *)((void *)&sval);
-    if (t==val_int)
-	*n = itoString(v.ival);
-    else if (t==val_float)
-	*n = ftoString(v.fval);
-
-    return sval;
-}
-
-EST_read_status load_ogi_label(EST_String filename, EST_Stream &s)
+EST_read_status load_ogi_label(const EST_String &filename, EST_Stream &s)
 {
 // This function reads OGI style label files. The start, end
 // time and names of the labels are mandatory. 
@@ -146,15 +106,14 @@ EST_read_status load_ogi_label(EST_String filename, EST_Stream &s)
     return format_ok;
 }
 
-EST_read_status load_esps_label(EST_String filename, EST_Stream &s, 
-				EST_String comment)
+EST_read_status load_esps_label(const EST_String &filename, EST_Stream &s)
 {
 // This function reads xlabel (esps label) style label files. The end
 // time and names of the labels are mandatory. Everything after the
 // name is put in a list, where each item in the list corresponds to a
 // chunk of text between two separators (by default a ";").
     EST_TokenStream ts;
-    (void)comment;
+    EST_String t;
     
     if (((filename == "-") ? ts.open(cin) : ts.open(filename)) != 0)
     {
@@ -174,16 +133,20 @@ EST_read_status load_esps_label(EST_String filename, EST_Stream &s,
         cerr << "couldn't find header in label file " << filename << endl;
         return wrong_format;
     }
-    ts.get();      // Skip the header "#"
+    t = ts.get();
+
+    if (!ts.eoln()) // make anything after # the stream name
+	s.set_stream_name(ts.get().string());
 
     read_esps_portion(ts, s);
     s.set_name(filename);
     ts.close();
+
     return format_ok;
 }
 
 
-EST_read_status load_words_label(EST_String filename, EST_Stream &s)
+EST_read_status load_words_label(const EST_String &filename, EST_Stream &s)
 {
 // This function reads label files in the form of simple word strings 
 // with no timing information.
@@ -207,62 +170,35 @@ EST_read_status load_words_label(EST_String filename, EST_Stream &s)
     return format_ok;
 }
 
-void parse_fields(const EST_String &s, EST_StrList &l)
-{
-    EST_TokenStream ts;
-    EST_String t;
-    ts.open_string(s);
-    ts.set_WhiteSpaceChars(";");
-    
-    while (!ts.eof())
-	l.append(ts.get().string());
-    
-    ts.close();
-}
-
 EST_read_status read_esps_portion(EST_TokenStream &ts, EST_Stream &s)
 {
-    EST_Stream_Item item;
-    EST_String name;
+    EST_String name, key, val;
+
+    ts.set_SingleCharSymbols(";");
+    ts.set_quotes('"','\\');
     
     while(!ts.eof() && (ts.peek() != "#"))
     {
-	item.init(s.stream_name());
-	item.set_end(atof(ts.get().string())); // time
-	ts.get(); // ignore that all important color
+	EST_Stream_Item *item = s.append();
+
+	item->set_end(atof(ts.get().string())); // time
+	ts.get();		// ignore that all important color
 	
 	for (name = ""; (!ts.eoln()) && (ts.peek() != ";"); )
 	{
 	    EST_Token &t = ts.get();
-	    if (name.length() > 0)     // preserve internal whitespace
+	    if (name.length() > 0)	// preserve internal whitespace
 		name += t.whitespace();  
 	    name += t.string();
 	}
 	
-	item.set_name(name);
+	item->set_name(name);
 	
 	if (ts.peek().string() == ";") // absorb separator
-	    ts.get();
-
-	while (!ts.eoln())
 	{
-	    EST_String field;
-	    for (field=""; ((ts.peek() != ";") &&
-			    (!ts.eoln()));)
-	    {
-		EST_Token &t = ts.get();
-		field += t.whitespace()+t.prepunctuation();
-		if (ts.quoted_mode())
-		    field += quote_string(t.string());
-		else
-		    field += t.string();
-                field += t.punctuation();		
-	    }
-	    item.fields.append(field);
-	    if (ts.peek() == ";")
-		ts.get();  // skip ";"
+	    ts.get();
+	    item->f.load(ts);
 	}
-	s.append(item);
     }
     return format_ok;
 }    
@@ -288,18 +224,19 @@ EST_read_status read_label_portion(EST_TokenStream &ts, EST_Stream &s, int sampl
 	
 	item.set_end((float)hend/(float)sample); // time
 	item.set_name(ts.get().string());	 // name
-
+	
 	if (!ts.eoln())
-	    StringtoStrList(ts.get_upto_eoln().string(), item.fields);
+	    item.f.set("lab_fields",ts.get_upto_eoln().string());
 	
 	s.append(item);
     }
-
+    
     //    return ts.eof() ? format_ok : misc_read_error;
     return format_ok;
 }    
 
-EST_read_status load_sample_label(EST_String filename, EST_Stream &s, int sample)
+EST_read_status load_sample_label(const EST_String &filename, 
+				  EST_Stream &s, int sample)
 {
     EST_TokenStream ts;
     
@@ -323,13 +260,14 @@ EST_read_status load_sample_label(EST_String filename, EST_Stream &s, int sample
     }
     s.set_name(filename);
     return format_ok;
-
+    
 }
 
-EST_write_status save_label_esps(EST_String filename, EST_Stream &a, int num)
+EST_write_status save_label_esps(const EST_String &filename, 
+				 const EST_Stream &a, 
+				 const EST_String &features)
 {
     EST_Stream_Item *ptr;
-    EST_TBI *iptr;
     
     ostream *outf;
     if (filename == "-")
@@ -337,14 +275,16 @@ EST_write_status save_label_esps(EST_String filename, EST_Stream &a, int num)
     else
 	outf = new ofstream(filename);
     
-    if (outf == 0)  {
-	cerr << "Can't open label output file " << filename << endl;
-	return misc_write_error;
+    if (!(*outf))  
+    {
+	cerr << "save_label_epsp: can't open label output file \"" << 
+	    filename << "\"" << endl;
+	return write_fail;
     }
     
     *outf << "separator ;\n";
     *outf << "nfields 1\n";
-    *outf << "#\n";
+    *outf << "# " << a.stream_name() << endl;
     
     for (ptr = a.head(); ptr != 0; ptr = next(ptr))
     {
@@ -355,23 +295,24 @@ EST_write_status save_label_esps(EST_String filename, EST_Stream &a, int num)
 	//	outf->fill('0');
 	*outf << ptr->end();
 	
-	*outf << " 26 \t" << ptr->name();
+	*outf << " 26 \t" << ptr->name() << " ; ";
 	
-	if (num == 1)
+	if (features == "tilt") 
 	{
-	    *outf << "; a" << ptr->addr();
-	    for (iptr = ptr->links.head(); iptr != 0; iptr = next(iptr))
-		*outf << (ptr->links(iptr).length() ? "; " : "")
-		    << ptr->links(iptr);
+	    EST_String tiltfeats = ptr->f("tilt");
+	    if (tiltfeats == "0")
+		cerr << "stream_io.cc:save_label_esps no tilt feature" << endl;
+	    else
+		*outf << "tilt: " << tiltfeats;
 	}
-	else if (num == 2)
-	    for (EST_TBI *p = ptr->fields.head(); p; p = next(p))
-		*outf << "; " << ptr->fields(p);
-
-	else if (num == 3)
-	    for (EST_TBI *p = ptr->features.list.head(); p; p = next(p))
-		*outf << "; " << ptr->features.list(p).k << ": " << ptr->features.list(p).v;
-
+	else if (features == "Features")
+	    ptr->f.save(*outf);
+	
+	else if (features != "None")
+	{
+	    cerr << "stream_io.cc:save_label_esps unknown feature type to be saved" << endl;
+	}
+	
 	*outf << endl;
     }
     
@@ -380,7 +321,8 @@ EST_write_status save_label_esps(EST_String filename, EST_Stream &a, int num)
     return write_ok;
 }
 
-EST_write_status save_label_htk(EST_String filename, EST_Stream &a)
+EST_write_status save_label_htk(const EST_String &filename, 
+				const EST_Stream &a)
 {
     EST_Stream_Item *ptr;
     
@@ -390,9 +332,11 @@ EST_write_status save_label_htk(EST_String filename, EST_Stream &a)
     else
 	outf = new ofstream(filename);
     
-    if (outf == 0)  {
-	cerr << "Can't open label output file " << filename << endl;
-	return misc_write_error;
+    if (!(*outf))
+    {
+	cerr << "save_label_htk: can't open label output file \"" << 
+	    filename << "\"" << endl;
+	return write_fail;
     }
     
     outf->precision(6);
@@ -412,7 +356,8 @@ EST_write_status save_label_htk(EST_String filename, EST_Stream &a)
     return write_ok;
 }
 
-EST_write_status save_label_spn(EST_String filename, EST_Stream &a)
+EST_write_status save_label_spn(const EST_String &filename, 
+				const EST_Stream &a)
 {
     EST_Stream_Item *ptr;
     
@@ -422,9 +367,11 @@ EST_write_status save_label_spn(EST_String filename, EST_Stream &a)
     else
 	outf = new ofstream(filename);
     
-    if (outf == 0)  {
-	cerr << "Can't open label output file " << filename << endl;
-	return misc_write_error;
+    if (!(*outf))
+    {
+	cerr << "save_label_spn: can't open label output file \"" 
+	    << filename << "\"" << endl;
+	return write_fail;
     }
     
     ptr = a.head();
@@ -460,7 +407,9 @@ EST_write_status save_label_spn(EST_String filename, EST_Stream &a)
     return write_ok;
 }
 
-EST_write_status save_label_names(EST_String filename, EST_Stream &a, int n)
+EST_write_status save_label_names(const EST_String &filename, 
+				  const EST_Stream &a, 
+				  const EST_String &features)
 {
     EST_Stream_Item *ptr;
     
@@ -470,21 +419,22 @@ EST_write_status save_label_names(EST_String filename, EST_Stream &a, int n)
     else
 	outf = new ofstream(filename);
     
-    if (outf == 0)  {
-	cerr << "Can't open label output file " << filename << endl;
+    if (!(*outf))  
+    {
+	cerr << "save_label_name: can't open label output file \"" 
+	    << filename << "\"" << endl;
 	return misc_write_error;
     }
     
     for (ptr = a.head(); next(ptr) != 0; ptr = next(ptr))
     {
 	*outf << ptr->name();
-	if (n == 0)
+	if ((features != "") && (features != "OneLine"))
 	    *outf << endl;
 	else
 	    *outf << " ";
     }
-
-//	*outf << ptr->name() << (n == 0 ? endl : " ");
+    
     *outf << ptr->name() << endl;
     
     if (outf != &cout)
@@ -492,7 +442,9 @@ EST_write_status save_label_names(EST_String filename, EST_Stream &a, int n)
     return write_ok;
 }
 
-EST_write_status save_StreamList(EST_String filename, EST_StreamList &plist, int time, int path)
+EST_write_status save_StreamList(const EST_String &filename, 
+				 const EST_StreamList &plist, 
+				 int time, int path)
 {
     EST_TBI *p;
     EST_Stream_Item *ptr;
@@ -504,9 +456,11 @@ EST_write_status save_StreamList(EST_String filename, EST_StreamList &plist, int
     else
 	outf = new ofstream(filename);
     
-    if (outf == 0)  {
-	cerr << "Can't open MLF output file \"" << filename << "\"\n";
-	return misc_write_error;
+    if (!(*outf))
+    {
+	cerr << "save_StreamList: can't open MLF output file \"" 
+	    << filename << "\"\n";
+	return write_fail;
     }
     
     *outf << "#!MLF!#\n";	// MLF header/identifier
@@ -536,8 +490,9 @@ EST_write_status save_StreamList(EST_String filename, EST_StreamList &plist, int
     return write_ok;
 }    
 
-EST_write_status save_WordList(EST_String filename, EST_StreamList &plist, 
-			       int n)
+EST_write_status save_WordList(const EST_String &filename, 
+			       const EST_StreamList &plist, 
+			       int style)
 {
     EST_TBI *p;
     EST_Stream_Item *ptr;
@@ -548,22 +503,25 @@ EST_write_status save_WordList(EST_String filename, EST_StreamList &plist,
     else
 	outf = new ofstream(filename);
     
-    if (outf == 0)  {
-	cerr << "Can't open WordList output file \"" << filename << "\"\n";
-	return misc_write_error;
+    if (!(*outf))
+    {
+	cerr << "save:WordList: can't open WordList output file \"" 
+	    << filename << "\"\n";
+	return write_fail;
     }
     
     for (p = plist.head(); p != 0; p = next(p))
     {
 	for (ptr = plist(p).head(); next(ptr) != 0; ptr = next(ptr))
-	{	
+	{
 	    *outf << ptr->name();
-	    if (n == 0)
+	    if (style == 0)
 		*outf << endl;
 	    else
 		*outf << " ";
 	}
-	*outf << ptr->name() << endl;
+	if (ptr != 0)
+	    *outf << ptr->name() << endl;
     }
     
     if (outf != &cout)
@@ -571,24 +529,27 @@ EST_write_status save_WordList(EST_String filename, EST_StreamList &plist,
     return write_ok;
 }    
 
-EST_write_status save_ind_StreamList(EST_String filename, 
-				     EST_StreamList &plist, int n, int path)
+EST_write_status save_ind_StreamList(const EST_String &filename, 
+				     const EST_StreamList &plist, 
+				     const EST_String &features, 
+				     int path)
 {
     EST_TBI *p;
     EST_String outname;
     (void) filename;
-
+    
     for (p = plist.head(); p != 0; p = next(p))
     {
 	outname = path ? plist(p).name() : basename(plist(p).name());
-	if (plist(p).save(outname, "esps", n) != write_ok)
+	if (plist(p).save(outname, "esps", features) != write_ok)
 	    return misc_write_error;
     }
     
     return write_ok;
 }    
 
-EST_read_status load_StreamList(EST_String filename, EST_StreamList &plist)
+EST_read_status load_StreamList(const EST_String &filename, 
+				EST_StreamList &plist)
 {
     EST_TokenStream ts;
     EST_String fns, name;
@@ -608,29 +569,8 @@ EST_read_status load_StreamList(EST_String filename, EST_StreamList &plist)
 	cerr << "Not MLF file\n";
 	return wrong_format;
     }
-
+    
     while(!ts.eof())
-/*    {
-	s.clear();
-	plist.append(s);
-	p = plist.tail();
-	// put filename in as stream name. The filename is usually surrounded
-	// by quotes, so remove these.
-
-	fns = ts.get().string();
-	strip_quotes(fns);
-	plist(p).set_name(fns);
-
-    	if (read_label_portion(ts, plist(p), 10000000) == misc_read_error)
-	{
-	    cerr << "error: in reading MLF file\n";
-	    cerr << "section for file " << s.stream_name() << 
-		" at line " << ts.linenum() << " is badly formatted\n";
-	    
-	    return misc_read_error;
-	}
-    }
-*/
     {
 	s.clear();
 	// put filename in as stream name. The filename is usually surrounded
@@ -638,8 +578,7 @@ EST_read_status load_StreamList(EST_String filename, EST_StreamList &plist)
 	fns = ts.get().string();
 	strip_quotes(fns);
 	s.set_name(fns);
-
-//    	if (read_label_portion(ts, plist.item(i), 10000000) == misc_read_error)
+	
     	if (read_label_portion(ts, s, 10000000) == misc_read_error)
 	{
 	    cerr << "error: in reading MLF file\n";
@@ -650,25 +589,25 @@ EST_read_status load_StreamList(EST_String filename, EST_StreamList &plist)
 	}
 	plist.append(s);
     }
-
+    
     return format_ok;
 }
 
 // add evenly spaced dummny end values to Stream
-void pad_ends(EST_Stream &s, float length)
+static void pad_ends(EST_Stream &s, float length)
 {
     EST_Stream_Item *p;
     int i;
-
+    
     for (i = 0, p = s.head(); p; p = next(p), ++i)
-	 p->set_end(length * float(i)/float(s.length()));
+	p->set_end(length * float(i)/float(s.length()));
 }
 
 EST_read_status read_StreamList(EST_StreamList &plist, EST_StrList &files, EST_Option &al)
 {
     EST_Stream s;
     EST_TBI *p, *plp;
-
+    
     if (al.val("-itype", 0) == "mlf")
     {
 	if (load_StreamList(files.first(), plist) != format_ok)
@@ -683,7 +622,7 @@ EST_read_status read_StreamList(EST_StreamList &plist, EST_StrList &files, EST_O
 	    if (al.present("-itype"))
 	    {
 		if (plist(plp).load_type(files(p), al.val("-itype"), 
-				al.ival("-lf", 0)) != format_ok)
+					 al.ival("-lf", 0)) != format_ok)
 		    exit (-1);
 	    }
 	    else if (plist(plp).load(files(p)) != format_ok)
@@ -691,8 +630,8 @@ EST_read_status read_StreamList(EST_StreamList &plist, EST_StrList &files, EST_O
 	    plist(plp).set_stream_name(files(p));
 	    if ((al.val("-itype", 0) == "words") && (al.present("-length")))
 		pad_ends(s, al.fval("-length"));
-
+	    
 	}
-
+    
     return format_ok;
 }
