@@ -42,17 +42,18 @@ EST_read_status load_TList_of_StrVector(EST_TList<EST_StrVector> &w,
 					const EST_String &filename,
 					const int vec_len);
 
-static void print_results(EST_Stream &wstream);
-static void do_search(EST_Stream &wstream);
+static void print_results(EST_Relation &wstream);
+static bool do_search(EST_Relation &wstream);
 static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c);
-static EST_VTCandidate *vit_candlist(EST_Stream_Item &s);
+static EST_VTCandidate *vit_candlist(EST_Item &s);
+static void top_n_candidates(EST_VTCandidate* &all_c);
 static void load_vocab(const EST_String &vfile);
 
-static int add_word(EST_Stream &w, const EST_String &word, int pos);
+static void add_word(EST_Relation &w, const EST_String &word, int pos);
 
 static void load_wstream(const EST_String &filename,
 			 const EST_String &vfile,
-			 EST_Stream &w,
+			 EST_Relation &w,
 			 EST_Track &obs);
 
 static void load_given(const EST_String &filename,
@@ -65,7 +66,7 @@ static double find_extra_gram_prob(EST_VTPath *p,int *state, int time);
 static void get_history(EST_StrVector &history, EST_VTPath *p);
 static void fill_window(EST_StrVector &window,EST_StrVector &history,
 			EST_VTPath *p,const int time);
-static int is_a_special(EST_String &s, int &val);
+static int is_a_special(const EST_String &s, int &val);
 static int max_history=0;
 
 static EST_Ngrammar ngram;
@@ -75,6 +76,13 @@ static float lm_scale = 1.0;
 static float ob_scale = 1.0;
 static float ob_scale2 = 1.0;
 
+// pruning beam widths
+static float beam=-1;
+static float ob_beam=-1;
+static int n_beam = -1;
+
+static bool trace_on = FALSE;
+
 // always logs
 static double ob_log_prob_floor = SAFE_LOG_ZERO;  
 static double ob_log_prob_floor2 = SAFE_LOG_ZERO;  
@@ -96,12 +104,13 @@ int main(int argc, char **argv)
 {
     EST_StrList files;
     EST_Option al;
-    EST_Stream wstream;
+    EST_Relation wstream;
     double floor; // a temporary
 
-    parse_command_line2(argc, argv, 
+    parse_command_line(argc, argv, 
        EST_String("Usage:\n")+
        "viterbi  <options>\n"+
+       "Find the best path through a set of candidates.\n"+
        "-ngram <string>     Grammar file, required\n"+
        "-given <string>     ngram left contexts, per frame\n"+
        "-observes <string>  Observations file, Ascii Track, required\n"+
@@ -125,11 +134,15 @@ int main(int argc, char **argv)
        "                 after sentence end\n"+
        "-default_tags    use default tags of "+SENTENCE_START_MARKER+","
 			SENTENCE_END_MARKER+" and "+SENTENCE_END_MARKER+"\n"+
-       "                 respectively\n"+
+       "                 respectively\n\n"+
 
        "-observes2 <string> second observations (overlays first, ob_type must be same)\n"+
        "-ob_floor2 <float>  \n"+
-       "-ob_scale2 <float>  \n",
+       "-ob_scale2 <float>  \n\n"+
+       "-ob_prune  <float> observation pruning beam width (log) probability\n"+
+       "-n_prune   <int>   top-n pruning of observations\n"+
+       "-prune     <float> pruning beam width (log) probability\n"+
+       "-trace             show details of search as it proceeds\n",
 			files, al);
 
     if (al.present("-ngram"))
@@ -189,6 +202,33 @@ int main(int argc, char **argv)
     if (al.present("-prev_prev_tag"))
 	ppstring = al.val("-prev_prev_tag");
 
+    // pruning
+    if (al.present("-prune"))
+	beam = al.fval("-prune");
+    else
+	beam = -1; // no pruning
+
+    if (al.present("-ob_prune"))
+	ob_beam = al.fval("-ob_prune");
+    else
+	ob_beam = -1; // no pruning
+
+    if (al.present("-n_prune"))
+    {
+	n_beam = al.ival("-n_prune");
+	if(n_beam <= 0)
+	{
+	    cerr << "WARNING : " << n_beam;
+	    cerr << " is not a reasonable value for -n_prune !" << endl;
+	}
+    }
+    else
+	n_beam = -1; // no pruning
+    
+
+
+    if (al.present("-trace"))
+	trace_on = TRUE;
 
     // language model floor
     if (al.present("-lm_floor"))
@@ -255,14 +295,17 @@ int main(int argc, char **argv)
 	}
     }
 
-    do_search(wstream);
-    print_results(wstream);
+    if(do_search(wstream))
+	print_results(wstream);
+    else
+	cerr << "No path could be found." << endl;
+
     return 0;
 }
 
-static void print_results(EST_Stream &wstream)
+static void print_results(EST_Relation &wstream)
 {
-    EST_Stream_Item *s;
+    EST_Item *s;
     float pscore;
     EST_String predict;
     FILE *fd;
@@ -277,16 +320,17 @@ static void print_results(EST_Stream &wstream)
 
     for (s=wstream.head(); s != 0 ; s=next(s))
     {
-	predict = s->feature("best").string();
-	pscore = s->feature("best_score");
+	predict = s->f("best").string();
+	pscore = s->f("best_score");
 	fprintf(fd,"%s %f\n",(const char *)predict,pscore);
     }
 
     if (outfile != "")
 	fclose(fd);
+
 }
 
-static void do_search(EST_Stream &wstream)
+static bool do_search(EST_Relation &wstream)
 {
     // Apply Ngram to matrix of probs 
     int states;
@@ -294,16 +338,26 @@ static void do_search(EST_Stream &wstream)
     states = ngram.states();
     EST_Viterbi_Decoder vc(vit_candlist,vit_npath,states);
 
-    vc.initialise(wstream);
+    vc.initialise(&wstream);
+
+    if((beam > 0) || (ob_beam > 0))
+	vc.set_pruning_parameters(beam,ob_beam);
+
+    if(trace_on)
+    {
+	vc.turn_on_trace();
+	cerr << "Starting Viterbi search..." << endl;
+    }
+
     vc.search();
 
-    vc.result("best");  // adds fields to w with best values 
+    return vc.result("best");  // adds fields to w with best values 
 
 }
 
 static void load_wstream(const EST_String &filename,
 			 const EST_String &vfile, 
-			 EST_Stream &w,
+			 EST_Relation &w,
 			 EST_Track &obs)
 {
     // Load in vocab and probs into Stream (this isn't general)
@@ -350,7 +404,7 @@ static void load_given(const EST_String &filename,
     // set max history
     for (p = given.head(); p; p = next(p))
     {
-	for(i=0;i<given(p).num_points();i++)
+	for(i=0;i<given(p).length();i++)
 	    if(	is_a_special( given(p)(i), j) && (-j > max_history))
 		max_history = -j;
 	
@@ -376,20 +430,15 @@ static void load_vocab(const EST_String &vfile)
     ts.close();
 }
 
-static int add_word(EST_Stream &w, const EST_String &word, int pos)
+static void add_word(EST_Relation &w, const EST_String &word, int pos)
 {
-    static int addr=0;
-    EST_Stream_Item item;
+    EST_Item *item = w.append();
     
-    item.init("Word");
-    item.set_name(word);
-    item.set_feature("pos",pos);
-    item.set_addr(addr);
-    w.append(item);
-    return addr++;
+    item->set_name(word);
+    item->fset("pos",pos);
 } 
 
-static EST_VTCandidate *vit_candlist(EST_Stream_Item &s)
+static EST_VTCandidate *vit_candlist(EST_Item &s)
 {
     // Return a list of new candidates from this point 
     double prob=1.0,prob2=1.0;
@@ -399,7 +448,7 @@ static EST_VTCandidate *vit_candlist(EST_Stream_Item &s)
     EST_VTCandidate *all_c = 0;
     EST_VTCandidate *c;
 
-    observe = s.feature("pos");  // index for observations TRACK
+    observe = s.f("pos");  // index for observations TRACK
     for (i=0,p=vocab.head(); i < observations.num_channels(); i++,p=next(p))
     {
 	c = new EST_VTCandidate;
@@ -441,6 +490,13 @@ static EST_VTCandidate *vit_candlist(EST_Stream_Item &s)
 	c->s = &s;
 	all_c = c;
     }
+
+    if(n_beam > 0)
+    {
+	// N.B. this might be very time-consuming
+	top_n_candidates(all_c);
+    }
+
     return all_c;
 }
 
@@ -458,8 +514,8 @@ static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c)
     // are we using extra info ?
     if(using_given)
 	// time of candidate is
-	// c->s->feature("pos");
-	prob = find_extra_gram_prob(np,&np->state,c->s->feature("pos"));
+	// c->s->f("pos");
+	prob = find_extra_gram_prob(np,&np->state,c->s->f("pos"));
     else
 	prob = find_gram_prob(np,&np->state);
 
@@ -470,7 +526,7 @@ static EST_VTPath *vit_npath(EST_VTPath *p,EST_VTCandidate *c)
 
     lprob *= lm_scale;
 
-    np->set_feature("lscore",(c->score+lprob)); // simonk : changed prob to lprob
+    np->f.set("lscore",(c->score+lprob)); // simonk : changed prob to lprob
     if (p==0)
 	np->score = (c->score+lprob);
     else
@@ -492,18 +548,18 @@ static double find_gram_prob(EST_VTPath *p,int *state)
     {
 	if (pp != 0)
 	{
-	    window(i) = pp->c->name.string();
+	    window[i] = pp->c->name.string();
 	    pp = pp->from;
 	}
 	else if (f)
-	    window(i) = ppstring;
+	    window[i] = ppstring;
 	else
 	{
-	    window(i) = pstring;
+	    window[i] = pstring;
 	    f = TRUE;
 	}
     }
-    window(ngram.order()-1) = p->c->name.string();
+    window[ngram.order()-1] = p->c->name.string();
     const EST_DiscreteProbDistribution &pd = ngram.prob_dist(window);
     if (pd.samples() == 0)
 	prob = 0;
@@ -511,7 +567,7 @@ static double find_gram_prob(EST_VTPath *p,int *state)
 	prob = (double)pd.probability(p->c->name.string());
     
     for (i=0; i < ngram.order()-1; i++)
-	window(i) = window(i+1);
+	window[i] = window(i+1);
     ngram.predict(window,&nprob,state);
 
     return prob;
@@ -546,9 +602,9 @@ static double find_extra_gram_prob(EST_VTPath *p,int *state,int time)
     // shift history, adding latest item at 'end' (0)
     if(max_history>0)
     {
-	for(i=history.num_points()-1;i>0;i--)
-	    history(i) = history(i-1);
-	history(0) = p->c->name.string();
+	for(i=history.length()-1;i>0;i--)
+	    history[i] = history(i-1);
+	history[0] = p->c->name.string();
     }
 
     fill_window(window,history,p,time+1);
@@ -565,19 +621,19 @@ static void get_history(EST_StrVector &history, EST_VTPath *p)
 
     EST_VTPath *pp;
     int i,f=FALSE;
-    for (pp=p->from,i=0; i < history.num_points(); i++)
+    for (pp=p->from,i=0; i < history.length(); i++)
     {
 	
 	if (pp != 0)
 	{
-	    history(i) = pp->c->name.string();
+	    history[i] = pp->c->name.string();
 	    pp = pp->from;
 	}
 	else if (f)
-	    history(i) = ppstring;
+	    history[i] = ppstring;
 	else
 	{
-	    history(i) = pstring;
+	    history[i] = pstring;
 	    f = TRUE;
 	}
     }
@@ -603,34 +659,25 @@ static void fill_window(EST_StrVector &window,EST_StrVector &history,
     // also want vocab and grammar mismatch allowed !!!!!!
 
     // predictee
-    window(ngram.order()-1) = p->c->name.string();
+    window[ngram.order()-1] = p->c->name.string();
 
     // given info for this time
-    EST_StrVector *this_g = &(given.item(time)); // inefficient to count down a list
+    EST_StrVector *this_g = &(given.nth(time)); // inefficient to count down a list
 
 
-    //cerr << "window : ";
     for(i=0;i<ngram.order()-1;i++)
     {
 
 	if( is_a_special( (*this_g)(i), j))
-	{
-	    window(i) = history(-1-j); // j=-1 -> (0)   j=-2 -> (1)   etc.
-	    //cerr << "<" << j << "=" << window(i) << "> ";
-	}
+	    window[i] = history(-1-j); // j=-1 -> (0)   j=-2 -> (1)   etc.
 	else
-	{
-	    window(i) = (*this_g)(i);
-	    //cerr << window(i) << " ";
-	}
-	
+	    window[i] = (*this_g)(i);
     }
-    //cerr << endl;
 }
 
 
 
-static int is_a_special(EST_String &s, int &val)
+static int is_a_special(const EST_String &s, int &val)
 {
 
     // special is "<int>"
@@ -646,3 +693,61 @@ static int is_a_special(EST_String &s, int &val)
     }
     return FALSE;
 }
+
+static void top_n_candidates(EST_VTCandidate* &all_c)
+{
+    // keep the n most likely candidates
+    // avoiding a full sort of the (potentially long) list
+
+    EST_VTCandidate *top_c=NULL,*p,*q,*this_best, *prev_to_best;
+    int i;
+    if(n_beam < 1)
+	return; // do nothing
+
+    // here we assume big is always good
+    //if(!big_is_good)
+    //score_multiplier = -1;
+
+    for(i=0;i<n_beam;i++)
+    {
+
+	// head of the list is best guess
+	this_best=all_c;
+	prev_to_best=NULL;
+
+	// find best candidate in all_c
+	q=NULL;;
+	for(p=all_c;p!= NULL;q=p,p=p->next)
+	{
+	    //cerr << "item : " << p->score << endl;
+	    if(p->score > this_best->score)
+	    {
+		this_best = p;
+		prev_to_best=q;
+	    }
+	}
+	
+	if(this_best == NULL)
+	    break; // give up now - must have run out of candidates
+
+	// move best candidate over to new list
+	if(prev_to_best == NULL)
+	    // best was head of list
+	    all_c = this_best->next;
+	else
+	    // best was not head of all_c
+	    prev_to_best->next = this_best->next;
+
+	    this_best->next = top_c;
+	    top_c = this_best;
+    }
+
+    delete all_c;
+    all_c = top_c;
+
+/*
+    cerr << "Here are the top " << n_beam << " candidates" << endl;
+    for(p=all_c;p != NULL;p=p->next)
+	cerr << p->score << endl;
+*/
+}
