@@ -625,9 +625,6 @@ int read_next_wave_segment(EST_Wave &sig, Srpd_Op *paras, SEGMENT_ *p_seg)
     long offset;
     static int wave_pos;
 
-//    short *tmp_sig = (short *) sig.take_out(&p_samples);
-    short *tmp_sig = sig.data();
-
 /*    printf("size %d shift %d length %d\n", p_seg->size, p_seg->shift, p_seg->length);*/
 
     if (status == BEGINNING) 
@@ -666,8 +663,11 @@ int read_next_wave_segment(EST_Wave &sig, Srpd_Op *paras, SEGMENT_ *p_seg)
 	if (tracklen > 0) 
 	{
 	    offset = p_seg->shift;
-	    for (i = 0; i < p_seg->size; ++i)
-		p_seg->data[i] = tmp_sig[i + wave_pos];
+	    for (i = 0; (i < p_seg->size) && (i+wave_pos)<sig.num_samples();
+		 ++i)
+		p_seg->data[i] = sig.a(i + wave_pos,0);
+	    for ( ; i < p_seg->size; ++i)
+		p_seg->data[i] = 0;
 
 	    if (wave_pos <= sig.num_samples())
 	    {
