@@ -36,15 +36,19 @@
 #include <math.h>
 #include <stdlib.h>
 #include "EST_cutils.h"
-#include "EST_String.h"
-#include "EST_Stream.h"
 #include "EST_simplestats.h"
 #include "EST_sort.h"
 #include "EST_Track.h"
-#include "track_io.h"
+#include "EST_TrackFile.h"
 #include "EST_Option.h"
 #include "EST_track_aux.h"
 
+static inline int irint(float f) { return (int)(f+0.5); }
+static inline int irint(double f) { return (int)(f+0.5); }
+static inline int ifloor(float f) { return (int)(f); }
+
+float correlation(EST_Track &a, EST_Track &b, int cha, int chb);
+
 static int sorttest(const void *a, const void *b)
 { // for use with qsort C library function.
     float *c = (float *)a;
@@ -65,7 +69,7 @@ void track_smooth(EST_Track &c, float x, EST_String stype)
 
 void time_med_smooth(EST_Track &c, float x)
 {
-    if (c.space_type() != "FIXED")
+  if (!c.equal_space())
     {
 	cerr << "Error: Time smoothing can only operate on fixed contours\n";
 	return;
@@ -186,30 +190,37 @@ void normalise(EST_Track &tr)
     EST_FVector mean, sd;
     
     meansd(tr, mean, sd);
-    normalise(tr, mean, sd);
+    normalise(tr, mean, sd, -1.0, 1.0);
 }
 
 /* Normalise a list of tracks */
-void normalise(EST_TrackList &trlist, EST_FVector &mean, EST_FVector &sd)
+void normalise(EST_TrackList &trlist, EST_FVector &mean, EST_FVector &sd, 
+	       float upper, float lower)
 {
     for (EST_TBI *p = trlist.head(); p; p = next(p))
-	normalise(trlist(p), mean, sd);
+	normalise(trlist(p), mean, sd, upper, lower);
 }
 
 /* Normalise by subtracting the mean and dividing by TWICE the
    standard deviation. */
-void normalise(EST_Track &tr, EST_FVector &mean, EST_FVector &sd)
+void normalise(EST_Track &tr, EST_FVector &mean, EST_FVector &sd,
+	       float upper, float lower)
 {
     for (int i = 0; i < tr.num_channels(); ++i)
-	normalise(tr, mean(i), sd(i), i);
+	normalise(tr, mean(i), sd(i), i, upper, lower);
 }
 
-void normalise(EST_Track &tr, float mean, float sd, int channel)
+void normalise(EST_Track &tr, float mean, float sd, int channel,
+	       float upper, float lower)
 {
+    // This scales the data so that 2 standard deviations worth of values
+    // lie between upper and lower.
     int i;
+    cout << "upper = " << upper << " lower " << lower << endl;
     for (i = 0; i < tr.num_frames(); ++i)
 	if (!tr.track_break(i))
-	    tr.a(i, channel) = (tr.a(i, channel) - mean) / (sd * 2.0);
+	    tr.a(i, channel) = ((((tr.a(i, channel) - mean) / (4 *sd)) + 0.5)
+				* (upper -lower))  + lower; 
 }
 
 EST_Track differentiate(EST_Track &c, float samp_int)
@@ -236,9 +247,6 @@ EST_Track differentiate(EST_Track &c, float samp_int)
 	diff.t(i) = c.t(i) + (dist / 2.0);
     }
     
-    diff.amin = -100;
-    diff.amax = 100;
-    
     return diff;
 }
 
@@ -264,6 +272,47 @@ EST_Track difference(EST_Track &a, EST_Track &b)
     return diff;
 }
 
+EST_Track difference(EST_Track &a, EST_Track &b, int channel_a, int channel_b)
+{
+    int i;
+    
+    int size = Lof(a.num_frames(), b.num_frames());
+    EST_Track diff = a;
+    
+    for (i = 0; i < size; ++i)
+	diff.a(i, channel_a) = a.a(i, channel_a) - b.a(i, channel_b);
+    
+    return diff;
+}
+
+EST_Track difference(EST_Track &a, EST_Track &b, EST_String fname)
+{
+    int ch_a, ch_b;
+    EST_Track cor;
+
+    if (!a.has_channel(fname))
+    {
+	cerr << "Error: Couldn't find field named " << fname <<
+	    " in first Track\n";
+	return cor;
+    }
+
+    if (!b.has_channel(fname))
+    {
+	cerr << "Error: Couldn't find field named " << fname <<
+	    " in second Track\n";
+	return cor;
+    }
+
+    ch_a = a.channel_position(fname);
+    ch_b = b.channel_position(fname);
+
+    return difference(a, b, ch_a, ch_b);
+}
+
+/** Calculate the mead and standard deviation for a single channel of a track
+*/
+
 void meansd(EST_Track &tr, float &mean, float &sd, int channel)
 {
     float var;
@@ -285,6 +334,10 @@ void meansd(EST_Track &tr, float &mean, float &sd, int channel)
     sd = sqrt(var);
 }
 
+/** Calculate the root mean square error between the same channel in
+two tracks 
+@see abs_error, rms_error
+*/
 float rms_error(EST_Track &a, EST_Track &b, int channel)
 {
     int i;
@@ -313,7 +366,7 @@ float abs_error(EST_Track &a, EST_Track &b, int channel)
     return sum / size;
 }
 
-float correlation(EST_Track &a, EST_Track &b, int channel)
+float correlation(EST_Track &a, EST_Track &b, int channela, int channelb)
 {
     int i;
     int size = Lof(a.num_frames(), b.num_frames());
@@ -324,9 +377,9 @@ float correlation(EST_Track &a, EST_Track &b, int channel)
     for (i = 0; i < size; ++i)
 	if (a.val(i) && b.val(i))
 	{
-	    cout << a.a(i, channel) << " " << b.a(i, channel) << endl;
-	    predict = b.a(i, channel);
-	    real = a.a(i, channel);
+//	    cout << a.t(i) << " " << a.a(i, channela) << " " << b.a(i, channelb) << endl;
+	    predict = b.a(i, channelb);
+	    real = a.a(i, channela);
 	    x += predict;
 	    y += real;
 	    error = predict-real;
@@ -341,8 +394,8 @@ float correlation(EST_Track &a, EST_Track &b, int channel)
         (sqrt(xx.mean()-(x.mean()*x.mean())) *
 	 sqrt(yy.mean()-(y.mean()*y.mean())));
     
-    cout << xy.mean()  << " " << x.mean() << " " << y.mean() << " "
-        << xx.mean() << " " << yy.mean() << endl;
+//    cout << xy.mean()  << " " << x.mean() << " " << y.mean() << " "
+//        << xx.mean() << " " << yy.mean() << endl;
     
     cout << "RMSE " << sqrt(se.mean()) << " Correlation is " << cor 
         << " Mean (abs) Error " << e.mean() << " (" << e.stddev() << ")" 
@@ -358,7 +411,7 @@ void meansd(EST_Track &a, EST_FVector &m, EST_FVector &sd)
     sd.resize(a.num_channels());
     
     for (i = 0; i < a.num_channels(); ++i)
-	meansd(a, m(i), sd(i), i);
+	meansd(a, m[i], sd[i], i);
 }
 
 void meansd(EST_TrackList &tl, float &mean, float &sd, int channel)
@@ -372,11 +425,13 @@ void meansd(EST_TrackList &tl, float &mean, float &sd, int channel)
 
     for (p = tl.head(); p; p = next(p))
 	for (i = 0; i < tl(p).num_frames(); ++i)
+	{
 	    if (!tl(p).track_break(i))
 	    {
 		mean += tl(p).a(i, channel);
 		++n;
 	    }
+    }
 
     mean /= n;
 
@@ -395,9 +450,9 @@ void meansd(EST_TrackList &tl, EST_FVector &m, EST_FVector &sd)
     
     m.resize(tl.first().num_channels());
     sd.resize(tl.first().num_channels());
-    
+
     for (i = 0; i < tl.first().num_channels(); ++i)
-	meansd(tl, m(i), sd(i), i);
+	meansd(tl, m[i], sd[i], i);
 }
 
 EST_FVector rms_error(EST_Track &a, EST_Track &b)
@@ -414,7 +469,7 @@ EST_FVector rms_error(EST_Track &a, EST_Track &b)
     }
     e.resize(a.num_channels());
     for (i = 0; i < a.num_channels(); ++i)
-	e(i) = rms_error(a, b, i);
+	e[i] = rms_error(a, b, i);
     
     return e;
 }
@@ -433,7 +488,7 @@ EST_FVector abs_error(EST_Track &a, EST_Track &b)
     }
     e.resize(a.num_channels());
     for (i = 0; i < a.num_channels(); ++i)
-	e(i) = abs_error(a, b, i);
+	e[i] = abs_error(a, b, i);
     
     return e;
 }
@@ -452,7 +507,35 @@ EST_FVector correlation(EST_Track &a, EST_Track &b)
     }
     cor.resize(a.num_channels());
     for (i = 0; i < a.num_channels(); ++i)
-	cor(i) = correlation(a, b, i);
+	cor[i] = correlation(a, b, i, i);
+    
+    return cor;
+}
+
+EST_FVector correlation(EST_Track &a, EST_Track &b, EST_String fname)
+{
+    int ch_a, ch_b;
+    EST_FVector cor;
+
+    if (!a.has_channel(fname))
+    {
+	cerr << "Error: Couldn't find field named " << fname <<
+	    " in first Track\n";
+	return cor;
+    }
+
+    if (!b.has_channel(fname))
+    {
+	cerr << "Error: Couldn't find field named " << fname <<
+	    " in second Track\n";
+	return cor;
+    }
+
+    ch_a = a.channel_position(fname);
+    ch_b = b.channel_position(fname);
+
+    cor.resize(1);
+    cor[0] = correlation(a, b, ch_a, ch_b);
     
     return cor;
 }
@@ -462,12 +545,12 @@ EST_Track error(EST_Track &ref, EST_Track &test, int relax)
     int i, j, k, l;
     EST_Track diff;
     diff = ref;
-    int t;
+    float t;
     
     // relaxation allows an error to be ignored near boundaries. The
     // degree of relation specifies how many frames can be ignored.
     
-    int *r = new int[relax*3];
+    float *r = new float[relax*3];
     
     for (l = 0; l < ref.num_channels(); ++l)
 	for (i = 0; i < ref.num_frames(); ++i)
@@ -477,13 +560,15 @@ EST_Track error(EST_Track &ref, EST_Track &test, int relax)
 	    {
 		if (ref.a(i, l) > 0.5)
 		    r[k] = ((j < test.num_frames()) && (test.a(j, l)> 0.6)) ?1 
-			: 0;
+			: 0.5;
 		else
-		    r[k] = ((j < test.num_frames()) && (test.a(j, l)< 0.4)) ?1 
-			: 0;
-		
-		t |= r[k];
+		    r[k] = ((j < test.num_frames()) && (test.a(j, l)< 0.4)) ? -1 
+			: -0.5;
+
+		// fix for relaxation
+		t = r[k];
 	    }
+//	    cout << "ref: " << ref.a(i, l) << " test:" << test.a(i, l) << " error:" << t << endl;
 	    diff.a(i, l) = t;
 	}
     
@@ -491,60 +576,6 @@ EST_Track error(EST_Track &ref, EST_Track &test, int relax)
     return diff;
 }
 
-int track_divide(EST_TList<EST_Track> &mtfr, EST_Track &fv, EST_Stream &key)
-{
-    EST_Track a;
-    EST_Stream_Item  *k, t;
-    float kstart, length;
-    int i, j, l, n;
-    
-    mtfr.clear();
-    
-    if ((key.tail())->end() < (fv.t(fv.num_frames() - 1)))
-    {
-	cerr << "Key file must extend beyond end of EST_Track\n";
-	cerr << "key end: " << key.tail()->end() << " EST_Track end: " 
-	    << fv.t(fv.num_frames() - 1) << endl;
-	return -1;
-    }
-    
-    k = key.head();
-    a.set_name(k->name());
-    kstart = 0.0;
-    
-    length = k->end() - kstart;
-    n = (int)(length / (float) fv.shift()) + 2;
-    a.resize(n, fv.num_channels());
-    
-    for (i = 0, l = 0; i < fv.num_frames(); ++i, ++l)
-    {
-	for (j = 0; j < fv.num_channels(); ++j)
-	    a(l, j) = fv(i, j);
-	
-	if (fv.t(i) > k->end())
-	{
-	    a.set_num_frames(l + 1);
-	    mtfr.append(a);
-	    
-	    kstart = k->end();
-	    k = next(k);
-	    a.set_name(k->name());
-	    length = k->end() - kstart;
-	    n = (int)(length / (float) fv.shift()) + 2;
-	    //	    cout << "n frames: " << n << endl;
-	    a.resize(n, fv.num_channels());
-	    a.fill_time(fv.shift());
-	    
-	    //	for (j = 0; j < fv.order(); ++j)
-	    //	    a(0, j) = fv(i, j);
-	    l = -1;
-	}
-    }
-    a.set_num_frames(l);
-    mtfr.append(a);
-    return 0;
-}
-
 void align_to_track(EST_Track &tr, float &start, float &end)
 {
   int is, ie;
@@ -555,11 +586,12 @@ void align_to_track(EST_Track &tr, float &start, float &end)
   ie = tr.index(end);
 
   // cout << " indexes " << is << " " << ie << "\n";
-  
+
   start = tr.t(is);
   end   = tr.t(ie);
   // cout << " out " << start << " " << end << "\n";
 }
+
 void align_to_track(EST_Track &tr, int &start, int &end, int sample_rate)
 {
     float start_t = start/(float)sample_rate;
@@ -574,11 +606,105 @@ void align_to_track(EST_Track &tr, int &start, int &end, int sample_rate)
       end = (int)(  end_t*sample_rate + 0.5);
 }
 
+void move_to_frame_ends(EST_Track &tr, 
+			int &start, int &end, 
+			int sample_rate,
+			float offset)
+{
+    float start_t = start/(float)sample_rate;
+    float   end_t =   end/(float)sample_rate;
+    
+    // cout << "move " << start_t << " " << end_t << " " << sample_rate << "\n";
 
+    int is = tr.index(start_t-offset);
+    int ie = tr.index(end_t-offset);
 
-EST_Track extract(EST_Track &orig, float start, float end)
+    int start_s, start_c, start_e;
+    int end_s, end_c, end_e;
+
+    if (tr.has_channel(channel_length))
+    {
+	get_frame(tr, sample_rate, is, start_s, start_c, start_e);
+	get_frame(tr, sample_rate, ie, end_s, end_c, end_e);
+    }
+    else
+    {
+	start_s = (int)(tr.t(is) * sample_rate);
+	end_s = (int)(tr.t(ie) * sample_rate);
+    }
+  
+    start = start_s + (int)(offset*sample_rate + 0.5);
+    end = end_e   + (int)(offset*sample_rate + 0.5);
+}
+
+int nearest_boundary(EST_Track &tr, float time, int sample_rate, float offset)
+{
+    time -= offset;
+
+    float distance = 10000;
+
+    for (int i = 0; i < tr.num_frames(); ++i)
+    {
+	float start, center, end;
+
+	get_frame(tr, sample_rate, i, start, center, end);
+
+	// printf("nb %f: %d distance %f start %f\n", time, i, distance, start);
+	
+	if (fabs(start-time) > distance)
+	    return i-1;
+	distance = fabs(start-time);
+    }
+    
+    return  tr.num_frames();
+}
+
+void move_start(EST_Track &tr, float shift)
+{
+    for(int i=0; i<tr.num_frames(); i++)
+	tr.t(i) += shift;
+}
+
+void set_start(EST_Track &tr, float start)
+{
+    float shift = start - tr.t(0);
+    
+    move_start(tr, shift);
+}
+
+
+void extract2(EST_Track &orig, float start, float end, EST_Track &ret)
+{
+    int from, to;
+    int i, j;
+    from = orig.index(start);
+    to = orig.index_below(end);
+    
+    ret.resize(to - from, orig.num_channels());
+    
+    for (i = 0; i < ret.num_frames(); ++i)
+	for (j = 0; j < ret.num_channels(); ++j)
+	{
+	    ret.a(i, j) = orig.a(i + from, j);
+	    ret.t(i) = orig.t(i + from);
+	    if (orig.track_break(i + from))
+		ret.set_break(i);
+	    else
+		ret.set_value(i);
+	}
+    
+    
+    cout << "from " << from << " to " << to << endl;
+    cout << "times from " << orig.t(from) << " to " << orig.t(to) << endl;
+    
+    //    orig.sub_track(ret, from, to);
+    cout << ret.num_frames() << " " << ret.start() << " " << ret.end() << endl;
+    cout << "ret " << ret;
+}
+
+
+void extract(EST_Track &orig, float start, float end, EST_Track &ret)
 {
-    EST_Track ret;
     int new_num_frames;
     
     ret.copy_setup(orig);
@@ -603,48 +729,100 @@ EST_Track extract(EST_Track &orig, float start, float end)
 	else
 	    ret.set_value(i);
     }
-    return ret;
 }
 
-int get_order(const EST_Track &t)
+int get_order(const EST_Track &t, EST_CoefficientType type, int d)
 {
-  int order;
-  if (t.has_channel(channel_coef0))
-    if (t.has_channel(channel_coefN))
-      order = t.channel_position(channel_coefN) - t.channel_position(channel_coef0);
+    int order;
+    EST_ChannelType start_c = (EST_ChannelType)EST_CoefChannelId(type, d, 0);
+    EST_ChannelType end_c = (EST_ChannelType)EST_CoefChannelId(type, d, 1);
+    
+    if (t.has_channel(start_c))
+	if (t.has_channel(end_c))
+	    order = t.channel_position(end_c) - t.channel_position(start_c);
+	else
+	    order = t.num_channels()-t.channel_position(start_c)-1;
+    else
+	order=0;
+    return order;
+}
+
+int get_order(const EST_Track &tr)
+{
+    int order=0;
+    EST_CoefficientType t;
+    
+    for(t=cot_first; t <cot_free; t=(EST_CoefficientType)(t+1))
+	if ((order=get_order(tr,t))>0)
+	    return order;
+    
+    cout << "No coefficients in track\n";
+    return 0;
+}
+
+int sum_lengths(const EST_Track &t, 
+		int sample_rate,
+		int start_frame, int end_frame)
+{
+    (void)sample_rate;
+    int l=0;
+    
+    if (end_frame < 0)
+	end_frame = t.num_frames();
+    
+    if (t.has_channel(channel_length))
+	for(int i=start_frame; i<end_frame; i++)
+	    l += (int)t.a(i, channel_length);
     else
-      order = t.num_channels()-t.channel_position(channel_coef0)-1;
-  else
     {
-      cout << "No coefficients in track\n";
-      order=0;
+	cout << "no length channel";
     }
-  return order;
+    
+    return l;
 }
 
-// Note, these assumes a length channel
-int sum_lengths(const EST_Track &t, int start_frame, int end_frame)
+void get_start_positions(const EST_Track &t, int sample_rate, 
+			 EST_TBuffer<int> &pos)
 {
-  int l=0;
-
-  if (end_frame < 0)
-    end_frame = t.num_frames();
-
-  for(int i=start_frame; i<end_frame; i++)
-    l += (int)t.a(i, channel_length);
-
-  return l;
+    pos.ensure(t.num_frames());
+    
+    if (!t.has_channel(channel_length))
+    {
+	cout << "no length channel\n";
+	return;
+    }
+    
+    for(int i=0; i<t.num_frames(); i++)
+    {
+	int wstart, wcent, wend;
+	get_frame(t, sample_rate, i, wstart, wcent, wend);
+	pos[i] = wstart;
+	// cout << "frame " << i << " t " << t.t(i) << " sr " << sample_rate << " offset " << t.a(i,channel_offset) << " cent " << wcent << " pos " << wstart << "\n";
+    }
 }
 
-int *get_start_positions(const EST_Track &t)
+
+void extract(EST_Track &tr, EST_Option &al)
 {
-  int *pos = new int[t.num_frames()];
-  int p=0;
-
-  for(int i=0; i<t.num_frames(); p += (int)t.a(i, channel_length), i++)
-    pos[i] = p;
-
-  return pos;
+    int from, to;
+    EST_Track sub_track;
+    
+    if (al.present("-start"))
+	from = tr.index(al.fval("-start"));
+    else if (al.present("-from"))
+	from = al.ival("-from");
+    else
+	from = 0;
+    
+    if (al.present("-end"))
+	to = tr.index(al.fval("-end"));
+    else if (al.present("-to"))
+	to = al.ival("-to");
+    else
+	to = tr.num_frames() - 1;
+    
+    tr.sub_track(sub_track, from, to-from+1);
+    tr = sub_track;
 }
 
 void extract_channel(EST_Track &orig, EST_Track &nt, EST_IList &ch_list)
@@ -652,15 +830,19 @@ void extract_channel(EST_Track &orig, EST_Track &nt, EST_IList &ch_list)
     int new_ch, i, j, k;
     EST_TBI *p;
     new_ch = ch_list.length();
-
+    
     nt.clear();
     nt.copy_setup(orig);
     nt.resize(orig.num_frames(), new_ch);
-
+    
     for (i = 0, p = ch_list.head(); p; p = next(p), ++i)
     {
 	k = ch_list(p);
-	cout << "Extracting track " << k << endl;
+
+	// simonk - changed cout to cerr
+	// otherwise text appears in output file !
+	cerr << "Extracting track " << k << endl;
+
 	if (k >= orig.num_channels())
 	{
 	    cerr << "Tried to extract channel number " << k 
@@ -670,11 +852,10 @@ void extract_channel(EST_Track &orig, EST_Track &nt, EST_IList &ch_list)
 	}
 	for (j = 0; j < orig.num_frames(); ++j)
 	    nt.a(j, i) = orig.a(j, k);
-	nt.set_field_name(orig.field_name(k), i);
+	nt.set_channel_name(orig.channel_name(k), i);
     }
     for (j = 0; j < orig.num_frames(); ++j)
 	nt.t(j) = orig.t(j);
-
 }
 
 void ParallelTracks(EST_Track &a, EST_TrackList &list,const EST_String &style)
@@ -683,63 +864,170 @@ void ParallelTracks(EST_Track &a, EST_TrackList &list,const EST_String &style)
     // "styles". "0" means take the size of the first track in the list,
     // "1" means take the size of the longest as the number of frames in
     // the created track.
-    EST_TBI *p;
+    EST_TBI *p, *longest;
     int num_channels, num_frames;
     int i, j, k, n;
     
     a.clear();
     for (num_channels=0,p=list.head(); p; p=next(p))
 	num_channels += list(p).num_channels();
-
+    
     if (style == "first")
+    {
 	num_frames = list.first().num_frames();
+	longest = list.head();
+    }
     else
     {   
 	if (style != "longest")
 	    cerr << "EST_Track: unknown combine style \"" << style << 
 		"\" assuming longest" << endl;
-	for (num_frames = 0, p = list.head(); p; p = next(p))
+	for (num_frames = 0, longest = p = list.head(); p; p = next(p))
 	    if (num_frames < list(p).num_frames())
+	    {
 		num_frames = list(p).num_frames();
+		longest = p;
+	    }
     }
     
     a.resize(num_frames, num_channels);
-    a.fill_amp(0.0);
+    a.fill(0.0);
     
     for (k = 0, p = list.head(); p; p = next(p))
     {
 	n = Lof(num_frames, list(p).num_frames());
 	for (j = 0; j < list(p).num_channels(); ++j, ++k)
+	{
 	    for (i = 0; i < n; ++i)
 		a(i, k) = list(p).a(i, j);
+	    a.set_channel_name(list(p).channel_name(j), k);
+	}
+    }
+    // fill time with times from longest file.
+    for (i = 0; i < list(longest).num_frames(); ++i)
+	a.t(i) = list(longest).t(i);
+}
+
+void channel_to_time(EST_Track &tr, int channel, float scale)
+{  
+    
+    for(int i=0; i < tr.num_frames(); i++)
+    {
+	tr.t(i) = tr.a(i,channel) * scale;
+    }
+    tr.set_equal_space(FALSE);
+}
+
+void channel_to_time(EST_Track &tr, EST_ChannelType c, float scale)
+{
+    int channel = NO_SUCH_CHANNEL;
+    
+    if (tr.map() != NULL && (channel = (tr.map()->get(c)) != NO_SUCH_CHANNEL))
+    {
+	channel_to_time(tr, channel, scale);
+	return;
+    }
+    else
+    {
+	cerr << "no channel '" << EST_default_channel_names.name(c) << "' = " << c << "\n";
+	abort();
     }
 }
 
-int read_track(EST_Track &tr, const EST_String &in_file, EST_Option &al)
+void channel_to_time(EST_Track &tr, const EST_String c_name, float scale)
 {
+    for (int c=0; c<tr.num_channels(); c++)
+	if (tr.channel_name(c) == c_name)
+	{
+	    channel_to_time(tr, c, scale);
+	    return;
+	}
+    
+    cerr << "no channel named '" << c_name << "'\n";
+    abort();
+}
 
-    float ishift = 0;
-
-    if (al.present("ishift"))
-	ishift = al.fval("ishift",1);
-    else if (al.present("time_channel"))
-	ishift = 1.0;		// doesn't matter, will be reset by track
-    
-    if (tr.load(in_file, ishift) != format_ok)
-	return -1;
-    
-    tr.create_map();
-    
-    // cout << "f0 "<< tr.has_channel(channel_f0) << ".\n";
-    
-    if (al.present("time_channel") && tr.has_channel(al.sval("time_channel")))
+void channel_to_time_lengths(EST_Track &tr, int channel, float scale)
+{  
+    float tt=0;
+    for(int i=0; i < tr.num_frames(); i++)
     {
-	cout << " time from channel " << al.sval("time_channel") << "\n";
-	tr.channel_to_time(al.sval("time_channel"), al.fval("time_scale"));
+	// cout << "c_t_t " << i << " " << tt << "\n";
+	tr.t(i) = tt;
+	tt += tr.a(i,channel) * scale;
     }
+    tr.set_equal_space(FALSE);
+}
+
+void channel_to_time_lengths(EST_Track &tr, EST_ChannelType c, float scale)
+{
+    int channel = NO_SUCH_CHANNEL;
     
-    //    cout << tr;
-    return 0;
+    if (tr.map()!=NULL && (channel = tr.map()->get(c)) != NO_SUCH_CHANNEL)
+    {
+	channel_to_time_lengths(tr, channel, scale);
+	return;
+    }
+    else
+    {
+	cerr << "no channel '" << EST_default_channel_names.name(c) << "' = " << c << "\n";
+	abort();
+    }
+}
+
+void channel_to_time_lengths(EST_Track &tr, const EST_String c_name, float scale)
+{
+    for (int c=0; c<tr.num_channels(); c++)
+	if (tr.channel_name(c) == c_name)
+	{
+	    channel_to_time_lengths(tr, c, scale);
+	    return;
+	}
+    
+    cerr << "no channel named '" << c_name << "'\n";
+    abort();
+}
+
+void gc_track(void *t)
+{
+    delete (EST_Track *)t;
+}
+
+EST_String options_track_input(void)
+{
+    // The standard waveform input options 
+    return
+	EST_String("")+
+	    "-itype <string>  Input file type (optional).  If no type is\n"+
+		"                 specified type is automatically derived from\n"+
+		    "                 file's header. Supported types\n"+
+			"                 are: "+options_track_filetypes()+"\n"+
+			    "-c <string>      Contour type: F0, track\n"+
+				"-s <float>       Frame spacing of input in seconds\n";
+}
+
+EST_String options_subtrack(void)
+{
+    return
+	EST_String("")+
+	    "-start <float>   Extract contour starting at this time\n"+
+		"-end   <float>   Extract contour ending at this time\n"+
+		    "-from  <int>     Extract contour starting at this frame position\n"+
+			"-to    <int>     Extract contour ending at this frame position\n";
+}
+
+EST_String options_track_output(void)
+{
+    // The standard track output options 
+    return
+	EST_String("")+
+	    "-otype <string> {ascii}\n"+
+	"                 Output file type, if unspecified ascii is\n"+
+        "                 assumed, types are: "+options_track_filetypes()+
+	", label\n"+
+        "-S <float>       Frame spacing of output in seconds\n"+
+	"-o <ofile>      Output filename, defaults to stdout\n"+
+        "-color <string>  Colour of output track\n";
 }
 
 void track_info(EST_Track &t)
@@ -747,10 +1035,10 @@ void track_info(EST_Track &t)
     cout <<  t.name() << endl;
     cout << "Number of frames: " << t.num_frames() << endl;
     cout << "Number of channels: " << t.num_channels() << endl;
-    cout << "File type: " << t.file_type() << endl;
+    cout << "File type: " << EST_TrackFile::map.name(t.file_type()) << endl;
     cout << "Frame shift: " << t.shift() << endl;
     for (int i = 0; i < t.num_channels(); ++i)
-	cout << "Channel: " << i << ": " << t.field_name(i) << endl;
+	cout << "Channel: " << i << ": " << t.channel_name(i) << endl;
 }
 
 EST_String options_track_filetypes(void)
@@ -758,5 +1046,14 @@ EST_String options_track_filetypes(void)
     // Returns list of currently support track filetypes
     // Should be extracted from the list in EST_Track
     
-    return "esps, xmg, htk, ascii";
+    return EST_TrackFile::options_short();
 }
+
+EST_String options_track_filetypes_long(void)
+{
+    // Returns list of currently support track filetypes
+    // Should be extracted from the list in EST_Track
+    
+    return EST_TrackFile::options_supported();
+}
+
