@@ -34,6 +34,7 @@
 /*  Briemen et al. 1984                                                  */
 /*                                                                       */
 /*  Added decision list support, Feb 1997                                */
+/*  Added stepwise use of features, Oct 1997                             */
 /*                                                                       */
 /*=======================================================================*/
 
@@ -41,38 +42,65 @@
 #include <iostream.h>
 #include <fstream.h>
 #include <string.h>
-#include "EST.h"
+#include "EST_Token.h"
+#include "EST_FMatrix.h"
+#include "EST_multistats.h"
 #include "EST_Wagon.h"
+#include "EST_math.h"
 
-static void do_summary(WNode &tree,WDataSet &ds);
-static void test_tree_float(WNode &tree,WDataSet &ds);
-static void test_tree_class(WNode &tree,WDataSet &ds);
-static void test_tree_cluster(WNode &tree,WDataSet &dataset);
+WDataSet wgn_dataset;
+WDataSet wgn_test_dataset;
+Discretes wgn_discretes;
+EST_String wgn_test_file;
+EST_String wgn_desc_file;
+int wgn_min_cluster_size = 50;
+EST_FMatrix wgn_DistMatrix;
+int wgn_held_out = 0;
+int wgn_prune = TRUE;
+int wgn_quiet = FALSE;
+int wgn_verbose = FALSE;
+int wgn_csubset = FALSE;
+float wgn_float_range_split = 10;
+float wgn_balance = 0;
+
+static float do_summary(WNode &tree,WDataSet &ds,ostream *output);
+static float test_tree_float(WNode &tree,WDataSet &ds,ostream *output);
+static float test_tree_class(WNode &tree,WDataSet &ds,ostream *output);
+static float test_tree_cluster(WNode &tree,WDataSet &dataset, ostream *output);
 static int wagon_split(WNode &node);
 static WQuestion find_best_question(WVectorVector &dset);
 static void construct_binary_ques(int feat,WQuestion &test_ques);
 static float construct_float_ques(int feat,WQuestion &ques,WVectorVector &ds);
 static float construct_class_ques(int feat,WQuestion &ques,WVectorVector &ds);
 static void wgn_set_up_data(WVectorVector &data,const WVectorList &ds,int held_out,int in);
+static WNode *wagon_stepwise_find_next_best(float &bscore,int &best_feat);
 
 static int margin;
+char *wagon_version = "1.0.0 beta October 1997";
+
+Declare_TVector_Base_T(WVector *,NULL,NULL,WVectorP)
+
 
 #if defined(INSTANTIATE_TEMPLATES)
 // Instantiate class
 #include "../base_class/EST_TList.cc"
+#include "../base_class/EST_TVector.cc"
+
 template class EST_TList<WVector *>;
 template class EST_TItem<WVector *>;
-#include "../base_class/EST_TVector.cc"
-template class EST_TVector<WVector *>;
+
+Instantiate_TVector(WVector *)
+
 #endif
 
 void wgn_load_datadescription(EST_String fname)
 {
     // Load field description for a file
-    wgn_dataset.init(fname);
+    wgn_dataset.load_description(fname);
+    wgn_test_dataset.load_description(fname);
 }
 
-void wgn_load_dataset(EST_String fname)
+void wgn_load_dataset(WDataSet &dataset,EST_String fname)
 {
     // Read the data set from a filename.  One vector per line
     // Assume all numbers are numbers and non-nums are categorical
@@ -89,13 +117,26 @@ void wgn_load_dataset(EST_String fname)
 
     for ( ;!ts.eof(); )
     {
-	v = new WVector(wgn_dataset.width());
+	v = new WVector(dataset.width());
 	i = 0;
 	do 
 	{
-	    int type = wgn_dataset.dtype(i);
+	    int type = dataset.ftype(i);
 	    if (type == wndt_float)
-		v->set_flt_val(i,atof(ts.get().string()));
+	    {
+		// need to ensure this is not NaN or Infinity
+		float f = atof(ts.get().string());
+		if (finite(f))
+		    v->set_flt_val(i,f);
+		else
+		{
+		    cout << fname << ": bad float " << f
+			<< " in field " <<
+			dataset.feat_name(i) << " vector " << 
+			    dataset.samples() << endl;
+		    v->set_flt_val(i,0.0);
+		}
+	    }
 	    else if (type == wndt_binary)
 		v->set_int_val(i,atoi(ts.get().string()));
 	    else if (type == wndt_cluster)
@@ -111,86 +152,79 @@ void wgn_load_dataset(EST_String fname)
 		int n = wgn_discretes.discrete(type).name(s); 
 		if (n == -1)
 		{
-		    cout << "Bad value " << s << " in field " <<
-			wgn_dataset.feat_name(i) << " vector " << 
-			    wgn_dataset.samples() << endl;
+		    cout << fname << ": bad value " << s << " in field " <<
+			dataset.feat_name(i) << " vector " << 
+			    dataset.samples() << endl;
 		    n = 0;
 		}
 		v->set_int_val(i,n);
 	    }
 	    i++;
 	}
-	while (!ts.eoln() && i<wgn_dataset.width());
+	while (!ts.eoln() && i<dataset.width());
 	nvec ++;
-	if (i != wgn_dataset.width())
+	if (i != dataset.width())
 	{
-	    wagon_error(EST_String("Data vector contains ")+itoString(i)+
-			" parameters instead of "+
-			itoString(wgn_dataset.width()));
+	    wagon_error(fname+": data vector "+itoString(nvec)+" contains "
+			+itoString(i)+" parameters instead of "+
+			itoString(dataset.width()));
 	}
 	if (!ts.eoln())
 	{
-	    cerr << "Data vector " << nvec << 
+	    cerr << fname << ": data vector " << nvec << 
 		" contains too many parameters instead of " 
-		<< wgn_dataset.width() << endl;
+		<< dataset.width() << endl;
 	    wagon_error(EST_String("extra parameter(s) from ")+
 			ts.peek().string());
 	}
-	wgn_dataset.append(v);
+	dataset.append(v);
     }
 
-    cout << "Dataset of " << wgn_dataset.samples() << " vectors of " <<
-	wgn_dataset.width() << " parameters " << endl;
+    cout << "Dataset of " << dataset.samples() << " vectors of " <<
+	dataset.width() << " parameters from: " << fname << endl;
     ts.close();
 }
 
-static void summary_results(WNode &tree)
+float summary_results(WNode &tree,ostream *output)
 {
-    if (wgn_test_file != "")  // load in test data
-    {
-//	WDataSet ts;
-	wgn_load_datadescription(wgn_desc_file);
-	wgn_load_dataset(wgn_test_file);
-    }
-
-    do_summary(tree,wgn_dataset);
-}
-
-static void do_summary(WNode &tree,WDataSet &ds)
-{
-    if (wgn_dataset.dtype(0) == wndt_cluster)
-	test_tree_cluster(tree,ds);
-    else if (wgn_dataset.dtype(0) >= wndt_class)
-	test_tree_class(tree,ds);
+    if (wgn_test_dataset.samples() != 0)
+	return do_summary(tree,wgn_test_dataset,output);
     else
-	test_tree_float(tree,ds);
-
+	return do_summary(tree,wgn_dataset,output);
 }
 
-void wgn_build_tree()
+static float do_summary(WNode &tree,WDataSet &ds,ostream *output)
+{
+    if (wgn_dataset.ftype(0) == wndt_cluster)
+	return test_tree_cluster(tree,ds,output);
+    else if (wgn_dataset.ftype(0) >= wndt_class)
+	return test_tree_class(tree,ds,output);
+    else
+	return test_tree_float(tree,ds,output);
+}
+
+WNode *wgn_build_tree(float &score)
 {
     // Build init node and split it while reducing the impurity
     WNode *top = new WNode();
 
-    wgn_set_up_data(top->get_data(),wgn_dataset.get_data(),wgn_held_out,TRUE);
+    wgn_set_up_data(top->get_data(),wgn_dataset,wgn_held_out,TRUE);
 
     margin = 0;
     wagon_split(*top);  // recursively split data;
 
     if (wgn_held_out > 0)
     {
-	wgn_set_up_data(top->get_data(),wgn_dataset.get_data(),
-			wgn_held_out,FALSE);
+	wgn_set_up_data(top->get_data(),wgn_dataset,wgn_held_out,FALSE);
 	top->held_out_prune();
     }
 	
     if (wgn_prune)
 	top->prune();
 
-    *wgn_coutput << *top;
+    score = summary_results(*top,0);
 
-    summary_results(*top);
-    
+    return top;
 }
 
 static void wgn_set_up_data(WVectorVector &data,const WVectorList &ds,int held_out,int in)
@@ -198,104 +232,106 @@ static void wgn_set_up_data(WVectorVector &data,const WVectorList &ds,int held_o
     // Set data ommitting held_out percent if in is true
     // or only including 100-held_out percent if in is false
     int i,j;
-    EST_TBI *d;
+    EST_Litem *d;
 
-    if (!in)
-	cout << "Doing held out data stage\n";
     // Make it definitely big enough
     data.resize(ds.length());
     
     for (j=i=0,d=ds.head(); d != 0; d=next(d),j++)
     {
 	if ((in) && ((j%100) >= held_out))
-	    data(i++) = ds(d);
+	    data[i++] = ds(d);
 //	else if ((!in) && ((j%100 < held_out)))
-//	    data(i++) = ds(d);
+//	    data[i++] = ds(d);
 	else if (!in)
-	    data(i++) = ds(d);
+	    data[i++] = ds(d);
 //	if ((in) && (j < held_out))
-//	    data(i++) = ds(d);	    
+//	    data[i++] = ds(d);	    
 //	else if ((!in) && (j >=held_out))
-//	    data(i++) = ds(d);	    
+//	    data[i++] = ds(d);	    
     }
-    // make it the actual size
-    data.resize(i);
+    // make it the actual size, but don't reset values
+    data.resize(i,1);  
 }
 
-static void test_tree_class(WNode &tree,WDataSet &dataset)
+static float test_tree_class(WNode &tree,WDataSet &dataset,ostream *output)
 {
     // Test tree against data to get summary of results
     EST_StrStr_KVL pairs;
     EST_StrList lex;
-    EST_TBI *p;
-    WVectorList ds;
+    EST_Litem *p;
     EST_String predict,real;
     int i,type;
+    float correct=0,total=0;
 
-    ds = dataset.get_data();
-    for (p=ds.head(); p != 0; p=next(p))
+    for (p=dataset.head(); p != 0; p=next(p))
     {
-	predict = tree.predict((*ds(p)));
-	type = dataset.dtype(0);
-	real = wgn_discretes[type].name(ds(p)->get_int_val(0));
+	predict = tree.predict((*dataset(p)));
+	type = dataset.ftype(0);
+	real = wgn_discretes[type].name(dataset(p)->get_int_val(0));
+	if (real == predict)
+	    correct++;
+	total++;
 	pairs.add_item(real,predict,1);
     }
-    for (i=0; i<wgn_discretes[dataset.dtype(0)].size(); i++)
-	lex.append(wgn_discretes[dataset.dtype(0)].name(i));
+    for (i=0; i<wgn_discretes[dataset.ftype(0)].length(); i++)
+	lex.append(wgn_discretes[dataset.ftype(0)].name(i));
 
     const EST_FMatrix &m = confusion(pairs,lex);
-    print_confusion(m,pairs,lex);
     
+    if (output != NULL)
+	print_confusion(m,pairs,lex);  // should be to output not stdout
+
+    return (float)correct/(float)total;
 }
 
-static void test_tree_cluster(WNode &tree,WDataSet &dataset)
+static float test_tree_cluster(WNode &tree,WDataSet &dataset,ostream *output)
 {
     // Test tree against data to get summary of results for cluster trees
-    WVectorList ds;
     WNode *leaf;
     int real;
     int right_cluster=0;
     EST_SuffStats ranking, meandist;
-    EST_TBI *p;
+    EST_Litem *p;
 
-    ds = dataset.get_data();
-    for (p=ds.head(); p != 0; p=next(p))
+    for (p=dataset.head(); p != 0; p=next(p))
     {
-	leaf = tree.predict_node((*ds(p)));
-	real = ds(p)->get_int_val(0);
+	leaf = tree.predict_node((*dataset(p)));
+	real = dataset(p)->get_int_val(0);
 	meandist += leaf->get_impurity().cluster_distance(real);
 	right_cluster += leaf->get_impurity().in_cluster(real);
 	ranking += leaf->get_impurity().cluster_ranking(real);
     }
 
-    // Want number in right class, mean distance in sds, mean ranking
-    if (wgn_coutput != &cout)   // save in output file
-	*wgn_coutput << ";; Right cluster " << right_cluster << " (" <<
-	    (int)(100.0*(float)right_cluster/(float)ds.length()) << 
+    if (output != NULL)
+    {
+	// Want number in right class, mean distance in sds, mean ranking
+	if (output != &cout)   // save in output file
+	    *output << ";; Right cluster " << right_cluster << " (" <<
+		(int)(100.0*(float)right_cluster/(float)dataset.length()) << 
+		    "%) mean ranking " << ranking.mean() << " mean distance "
+			<< meandist.mean() << endl;
+	cout << "Right cluster " << right_cluster << " (" <<
+	    (int)(100.0*(float)right_cluster/(float)dataset.length()) << 
 		"%) mean ranking " << ranking.mean() << " mean distance "
 		    << meandist.mean() << endl;
-    cout << "Right cluster " << right_cluster << " (" <<
-	(int)(100.0*(float)right_cluster/(float)ds.length()) << 
-	    "%) mean ranking " << ranking.mean() << " mean distance "
-		<< meandist.mean() << endl;
-
+    }
 
+    return 0;  // not sure what this should be
 }
 
-static void test_tree_float(WNode &tree,WDataSet &dataset)
+static float test_tree_float(WNode &tree,WDataSet &dataset,ostream *output)
 {
     // Test tree against data to get summary of results FLOAT
-    EST_TBI *p;
-    WVectorList ds;
+    EST_Litem *p;
     float predict,real;
     EST_SuffStats x,y,xx,yy,xy,se,e;
     double cor,error;
 
-    ds = dataset.get_data();
-    for (p=ds.head(); p != 0; p=next(p))
+    for (p=dataset.head(); p != 0; p=next(p))
     {
-	predict = tree.predict((*ds(p)));
-	real = ds(p)->get_flt_val(0);
+	predict = tree.predict((*dataset(p)));
+	real = dataset(p)->get_flt_val(0);
 	x += predict;
 	y += real;
 	error = predict-real;
@@ -306,22 +342,40 @@ static void test_tree_float(WNode &tree,WDataSet &dataset)
 	xy += predict*real;
     }
 
-    cor = (xy.mean() - (x.mean()*y.mean()))/
-	(sqrt(xx.mean()-(x.mean()*x.mean())) *
-	 sqrt(yy.mean()-(y.mean()*y.mean())));
+    // Pearson's product moment correlation coefficient
+//    cor = (xy.mean() - (x.mean()*y.mean()))/
+//	(sqrt(xx.mean()-(x.mean()*x.mean())) *
+//	 sqrt(yy.mean()-(y.mean()*y.mean())));
+    // Because when the variation is X is very small we can
+    // go negative, thus cause the sqrt's to give FPE
+    double v1 = xx.mean()-(x.mean()*x.mean());
+    double v2 = yy.mean()-(y.mean()*y.mean());
 
-    if (wgn_coutput != &cout)   // save in output file
-	*wgn_coutput 
-	    << ";; RMSE " << ftoString(sqrt(se.mean()),4,1)
+    double v3 = v1*v2;
+
+    if (v3 <= 0)
+	// happens when there's very little variation in x
+	cor = 0;
+    else
+	cor = (xy.mean() - (x.mean()*y.mean()))/ sqrt(v3);
+    
+    if (output != NULL)
+    {
+	if (output != &cout)   // save in output file
+	    *output 
+		<< ";; RMSE " << ftoString(sqrt(se.mean()),4,1)
+		<< " Correlation is " << ftoString(cor,4,1)
+		<< " Mean (abs) Error " << ftoString(e.mean(),4,1)
+		<< " (" << ftoString(e.stddev(),4,1) << ")" << endl;
+	
+	cout << "RMSE " << ftoString(sqrt(se.mean()),4,1)
 	    << " Correlation is " << ftoString(cor,4,1)
 	    << " Mean (abs) Error " << ftoString(e.mean(),4,1)
 	    << " (" << ftoString(e.stddev(),4,1) << ")" << endl;
-	
-    cout << "RMSE " << ftoString(sqrt(se.mean()),4,1)
-	<< " Correlation is " << ftoString(cor,4,1)
-	<< " Mean (abs) Error " << ftoString(e.mean(),4,1)
-	<< " (" << ftoString(e.stddev(),4,1) << ")" << endl;
-    
+    }
+
+    return cor;  // should really be % variance, I think
+//    return -sqrt(se.mean());  // should really be % variance, I think
 }
 
 static int wagon_split(WNode &node)
@@ -329,7 +383,6 @@ static int wagon_split(WNode &node)
     // Split given node (if possible)
     WQuestion q;
     WNode *l,*r;
-    int i;
 
     node.set_impurity(WImpurity(node.get_data()));
     q = find_best_question(node.get_data());
@@ -342,8 +395,9 @@ static int wagon_split(WNode &node)
 	wgn_find_split(q,node.get_data(),l->get_data(),r->get_data());
 	node.set_subnodes(l,r);
 	node.set_question(q);
-	if (!wgn_quiet)
+	if (wgn_verbose)
 	{
+	    int i;
 	    for (i=0; i < margin; i++)
 		cout << " ";
 	    cout << q << endl;
@@ -357,8 +411,9 @@ static int wagon_split(WNode &node)
     }
     else
     {
-	if (!wgn_quiet)
+	if (wgn_verbose)
 	{
+	    int i;
 	    for (i=0; i < margin; i++)
 		cout << " ";
 	    cout << "stopped samples: " << node.samples() << " impurity: " 
@@ -377,11 +432,11 @@ void wgn_find_split(WQuestion &q,WVectorVector &ds,
     y.resize(q.get_yes());
     n.resize(q.get_no());
     
-    for (iy=in=i=0; i < ds.size(); i++)
+    for (iy=in=i=0; i < ds.n(); i++)
 	if (q.ask(*ds(i)) == TRUE)
-	    y(iy++) = ds(i);
+	    y[iy++] = ds(i);
 	else
-	    n(in++) = ds(i);
+	    n[in++] = ds(i);
 
 }
 
@@ -397,23 +452,25 @@ static WQuestion find_best_question(WVectorVector &dset)
     // test each feature with each possible question
     for (i=1;i < wgn_dataset.width(); i++)
     {
-	if (wgn_dataset.dtype(i) == wndt_binary)
+	if (wgn_dataset.ignore(i) == TRUE)
+	    tscore = WGN_HUGE_VAL;     // ignore this feature this time
+	else if (wgn_dataset.ftype(i) == wndt_binary)
 	{
 	    construct_binary_ques(i,test_ques);
 	    tscore = wgn_score_question(test_ques,dset);
 	}
-	else if (wgn_dataset.dtype(i) == wndt_float)
+	else if (wgn_dataset.ftype(i) == wndt_float)
 	{
 	    tscore = construct_float_ques(i,test_ques,dset);
 	}
-	else if (wgn_dataset.dtype(i) == wndt_ignore)
-	    tscore = WGN_HUGE_VAL;
-	else if (wgn_csubset && (wgn_dataset.dtype(i) >= wndt_class))
+	else if (wgn_dataset.ftype(i) == wndt_ignore)
+	    tscore = WGN_HUGE_VAL;    // always ignore this feature
+	else if (wgn_csubset && (wgn_dataset.ftype(i) >= wndt_class))
 	{
 	    wagon_error("subset selection temporarily deleted");
 //	    tscore = construct_class_ques_subset(i,test_ques,dset);
 	}
-	else if (wgn_dataset.dtype(i) >= wndt_class)
+	else if (wgn_dataset.ftype(i) >= wndt_class)
 	    tscore = construct_class_ques(i,test_ques,dset);
 	if (tscore < bscore)
 	{
@@ -437,7 +494,7 @@ static float construct_class_ques(int feat,WQuestion &ques,WVectorVector &ds)
     test_q.set_oper(wnop_is);
     ques = test_q;
     
-    for (cl=0; cl < wgn_discretes[wgn_dataset.dtype(feat)].size(); cl++)
+    for (cl=0; cl < wgn_discretes[wgn_dataset.ftype(feat)].length(); cl++)
     {
 	test_q.set_operand1(EST_Val(cl));
 	tscore = wgn_score_question(test_q,ds);
@@ -466,10 +523,10 @@ static float construct_class_ques_subset(int feat,WQuestion &ques,
 
     ques.set_fp(feat);
     ques.set_oper(wnop_is);
-    float *scores = new float[wgn_discretes[wgn_dataset.dtype(feat)].size()];
+    float *scores = new float[wgn_discretes[wgn_dataset.ftype(feat)].length()];
     
     // Only do it for exists values
-    for (cl=0; cl < wgn_discretes[wgn_dataset.dtype(feat)].size(); cl++)
+    for (cl=0; cl < wgn_discretes[wgn_dataset.ftype(feat)].length(); cl++)
     {
 	ques.set_operand(flocons(cl));
 	scores[cl] = wgn_score_question(ques,ds);
@@ -482,7 +539,7 @@ static float construct_class_ques_subset(int feat,WQuestion &ques,
     {   // Only one so we know the best "split"
 	ques.set_oper(wnop_is);
 	ques.set_operand(car(order));
-	return scores[(int)FLONM(car(order))];
+	return scores[get_c_int(car(order))];
     }
 
     ques.set_oper(wnop_in);
@@ -527,7 +584,7 @@ static LISP sort_class_scores(int feat,float *scores)
     LISP items = NIL;
     LISP l;
 
-    for (i=0; i < wgn_discretes[wgn_dataset.dtype(feat)].size(); i++)
+    for (i=0; i < wgn_discretes[wgn_dataset.ftype(feat)].length(); i++)
     {
 	if (scores[i] != WGN_HUGE_VAL)
 	{
@@ -537,7 +594,7 @@ static LISP sort_class_scores(int feat,float *scores)
 	    {
 		for (l=items; l != NIL; l=cdr(l))
 		{
-		    if (scores[i] < scores[(int)FLONM(car(l))])
+		    if (scores[i] < scores[get_c_int(car(l))])
 		    {
 			CDR(l) = cons(car(l),cdr(l));
 			CAR(l) = flocons(i);
@@ -568,7 +625,7 @@ static float construct_float_ques(int feat,WQuestion &ques,WVectorVector &ds)
     ques = test_q;
 
     min = max = ds(0)->get_flt_val(feat);  /* set up some value */
-    for (d=0; d < ds.size(); d++)
+    for (d=0; d < ds.n(); d++)
     {
 	val = ds(d)->get_flt_val(feat);
 	if (val < min)
@@ -613,7 +670,7 @@ static float score_question_set(WQuestion &q, WVectorVector &ds, int ignorenth)
     int d;
     WVector *wv;
 
-    for (d=0; d < ds.size(); d++)
+    for (d=0; d < ds.n(); d++)
     {
 	if ((ignorenth < 2) ||
 	    (d%ignorenth != ignorenth-1))
@@ -632,40 +689,125 @@ static float score_question_set(WQuestion &q, WVectorVector &ds, int ignorenth)
     int min_cluster;
 
     if ((wgn_balance == 0.0) ||
-	(ds.size()/wgn_balance < wgn_min_cluster_size))
+	(ds.n()/wgn_balance < wgn_min_cluster_size))
 	min_cluster = wgn_min_cluster_size;
     else 
-	min_cluster = (int)(ds.size()/wgn_balance);
+	min_cluster = (int)(ds.n()/wgn_balance);
 
     if ((y.samples() < min_cluster) ||
 	(n.samples() < min_cluster))
 	return WGN_HUGE_VAL;
 
-    if (y.measure() == 0.0)
-	return n.measure();
-    else if (n.measure() == 0.0)
-       return y.measure();
-    else    
-	return (y.measure() + n.measure())/2.0;
-
+    return (y.measure() + n.measure());
 }
 
 float wgn_score_question(WQuestion &q, WVectorVector &ds)
 {
-    // This isn't v-fold cross validation !! Have to think more
-    // about this
-    EST_SuffStats a;
-    int v;
-    float best;
+    // This level of indirection was introduced for later expansion
 
-    for (v=0; v < wgn_vfold; v++)
+    return score_question_set(q,ds,1);
+}
+
+WNode *wagon_stepwise(float limit)
+{
+    // Find the best single features and incrementally add features
+    // that best improve result until it doesn't improve.
+    // This is bascially to automate what Kurt was doing in building
+    // trees, he then automated it in PERL and as it seemed to work
+    // I put it into wagon itself.
+    // This can be pretty computationally intensive
+    WNode *best = 0,*new_best = 0;
+    float bscore,best_score = 0;
+    int best_feat,i;
+    int nf = 1;
+
+    // Set all features to ignore
+    for (i=1; i < wgn_dataset.width(); i++)
+	wgn_dataset.set_ignore(i,TRUE);
+
+    for (i=1; i < wgn_dataset.width(); i++)
     {
-	best = score_question_set(q,ds,v);
-	a+=best;
+	if (wgn_dataset.ftype(i) == wndt_ignore)
+	{
+	    // This skips the round not because this has anything to
+	    // do with this feature being (user specificed) ignored
+	    // but because it indicates there is one less cycle that is
+	    // necessary
+	    continue;
+	}
+	new_best = wagon_stepwise_find_next_best(bscore,best_feat);
+
+	if ((bscore - (bscore * (limit/100))) <= best_score)
+	{
+	    // gone as far as we can
+	    delete new_best;
+	    break;
+	}
+	else
+	{
+	    best_score = bscore;
+	    delete best;
+	    best = new_best;
+	    wgn_dataset.set_ignore(best_feat,FALSE);
+	    if (!wgn_quiet)
+	    {
+		fprintf(stdout,"FEATURE    %d %s: %2.4f\n",
+			nf,
+			(const char *)wgn_dataset.feat_name(best_feat),
+			best_score);
+		fflush(stdout);
+		nf++;
+	    }
+	}
     }
 
-    if (wgn_vfold > 2)
-	best = score_question_set(q,ds,1);
+    return best;
+}
 
-    return a.mean();
+static WNode *wagon_stepwise_find_next_best(float &bscore,int &best_feat)
+{
+    // Find which of the currently ignored features will best improve
+    // the result
+    WNode *best = 0;
+    float best_score = 0;
+    int best_new_feat = -1;
+    int i;
+
+    for (i=1; i < wgn_dataset.width(); i++)
+    {
+	if (wgn_dataset.ftype(i) == wndt_ignore)
+	    continue; // user wants me to ignore this completely
+	else if (wgn_dataset.ignore(i) == TRUE)
+	{
+	    WNode *current;
+	    float score;
+
+	    // Allow this feature to participate
+	    wgn_dataset.set_ignore(i,FALSE);
+	    
+	    current = wgn_build_tree(score);
+
+	    if (score > best_score)
+	    {
+		best_score = score;
+		delete best;
+		best = current;
+		best_new_feat = i;
+//		fprintf(stdout,"BETTER FEATURE    %d %s: %2.4f\n",
+//			i,
+//			(const char *)wgn_dataset.feat_name(i),
+//			best_score);
+//		fflush(stdout);
+	    }
+	    else
+		delete current;
+
+	    // switch it off again
+	    wgn_dataset.set_ignore(i,TRUE);
+	}
+    }
+
+    bscore = best_score;
+    best_feat = best_new_feat;
+    return best;
 }
